/*
	$Id: Manager.java 4478 2008-09-05 15:15:54Z jari $

	Copyright (C) 2005 Nicklas Nordborg, Gregory Vincic
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg, Gregory Vincic
	Copyright (C) 2007 Jari Hakkinen, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	BASE is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
	02111-1307, USA.
*/

package net.sf.basedb.clients.migrate;

import net.sf.basedb.core.Application;

import java.sql.PreparedStatement;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Map;

/**
This class provides methods for logging, warning and stopping.

@author Gregory, Martin
@version 2.0
@base.modified $Date: 2008-09-05 17:15:54 +0200 (Fri, 05 Sep 2008) $
*/

abstract class Manager
{
	/**
	Log migration events.
	 */
	private static final org.apache.log4j.Logger log = org.apache.log4j.Logger.getLogger("net.sf.basedb.clients.migrate.Manager");
	private SimpleDateFormat sdf = new SimpleDateFormat("HH:mm:ss");

	/**
	PreparedStatements
	 */
	private Map<String, PreparedStatement> preparedStatements = null;

	public void setPreparedStatements(
			Map<String, PreparedStatement> preparedStatements)
	{
		this.preparedStatements = preparedStatements;
	}

	public Map<String, PreparedStatement> getPreparedStatements()
	{
		return preparedStatements;
	}
	
	/**
	@param statement SQL statement to prepare selected from configured
	       queryStatementsFile in migrate.properties.
	 */
	protected PreparedStatement prepareStatementFromFile(String statement)
	{
		PreparedStatement ps = preparedStatements.get(statement);
		if (ps == null)
		{
			log.error("Statement '" + statement + "' is null");
			stop();
		}
		return ps;
	}	
	
	/**
	Prints date without linebreak.
	 */
	protected void printDate()
	{
		System.out.print(sdf.format(new Date()) + " ");
	}
	
	/**
	Prints dated messages to standard output with linebreak.
	 */
	protected void print(String str)
	{
		printDate();
		System.out.println(str);
	}
	
	/**
		Stops the application
	 */
	protected void stop()
	{
		try
		{
			Application.stop();
		}
		catch (Exception e2)
		{
			log.error(e2);
		}
		log.info("Migration stopped");
		System.exit(1);
	}
}
