/**
	$Id: FixedMenuItemFactory.java 4478 2008-09-05 15:15:54Z jari $

	Copyright (C) Authors contributing to this file.

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.clients.web.extensions.menu;

import net.sf.basedb.clients.web.extensions.AbstractJspActionFactory;
import net.sf.basedb.util.Values;
import net.sf.basedb.util.extensions.InvokationContext;
import net.sf.basedb.util.extensions.xml.PathSetter;
import net.sf.basedb.util.extensions.xml.VariableSetter;

/**
	A simple implementation of a menu item action factory that 
	creates the same fixed menu item for all users 
	not using any context information. 
	<p>
	The menu is by default a regular menuitem and is both 
	visible and enabled. Use the setter method to change
	the properties. Changes to the properties are immediately
	visible in the menu items returned from the 
	{@link #getActions(InvokationContext)} method.	
	
	@author nicklas
	@version 2.7
	@base.modified $Date:2008-03-20 12:15:25 +0100 (Thu, 20 Mar 2008) $
*/
public class FixedMenuItemFactory 
	extends AbstractJspActionFactory<MenuItemAction>
	implements MenuItemAction 
{

	private MenuItemAction[] actions;
	
	private String menuId;
	private String submenuId;
	private String icon;
	private String onClick;
	private String style;
	private String title;
	private String tooltip;
	private boolean enabled;
	private boolean visible;
	private MenuType type;
	
	/**
		Create a new factory instance.
	*/
	public FixedMenuItemFactory()
	{
		actions = new MenuItemAction[] { this };
		enabled = true;
		visible = true;
		type = MenuType.MENUITEM;
	}
	
	/*
		From the ActionFactory interface
		--------------------------------
	*/
	@Override
	public MenuItemAction[] getActions(InvokationContext<? super MenuItemAction> context) 
	{
		return actions;
	}
	// ------------------------------------

	/*
		From the MenuItemAction interface
		--------------------------------
	*/
	@Override
	public String getIcon() 
	{
		return icon;
	}

	@Override
	public String getMenuId()
	{
		return menuId;
	}
	
	@Override
	public String getOnClick()
	{
		return onClick;
	}

	@Override
	public String getStyle() 
	{
		return style;
	}

	@Override
	public String getSubmenuId()
	{
		return submenuId;
	}

	@Override
	public String getTitle() 
	{
		return title;
	}

	@Override
	public String getTooltip() 
	{
		return tooltip;
	}

	@Override
	public MenuType getType() 
	{
		return type;
	}

	@Override
	public boolean isEnabled() 
	{
		return enabled;
	}

	@Override
	public boolean isVisible() 
	{
		return visible;
	}
	// ------------------------------------

	/*
		Setter methods that are called when the factory is
		initialised.
	*/
	@VariableSetter
	@PathSetter
	public void setIcon(String icon) 
	{
		this.icon = icon;
	}

	public void setMenuId(String menuId)
	{
		this.menuId = menuId;
	}
	
	@VariableSetter
	public void setOnClick(String onClick) 
	{
		this.onClick = onClick;
	}

	public void setStyle(String style) 
	{
		this.style = style;
	}

	public void setSubmenuId(String submenuId)
	{
		this.submenuId = submenuId;
	}

	public void setTitle(String title) 
	{
		this.title = title;
	}

	public void setTooltip(String tooltip) 
	{
		this.tooltip = tooltip;
	}

	public void setEnabled(boolean enabled) 
	{
		this.enabled = enabled;
	}

	public void setEnabled(String enabled) 
	{
		this.enabled = Values.getBoolean(enabled);
	}

	public void setVisible(boolean visible) 
	{
		this.visible = visible;
	}

	public void setVisible(String visible) 
	{
		this.visible = Values.getBoolean(visible);
	}
	
	public void setType(MenuType type)
	{
		this.type = type;
	}
	
	public void setType(String type)
	{
		this.type = MenuType.valueOf(type);
	}
	// ------------------------------------
	
}
