/**
	$Id: MenuItemBean.java 4478 2008-09-05 15:15:54Z jari $

	Copyright (C) Authors contributing to this file.

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.clients.web.extensions.menu;

/**
	A simple bean-like implementation of the {@link MenuItemAction}
	interface.
	
	@author nicklas
	@version 2.7
	@base.modified $Date: 2008-09-05 17:15:54 +0200 (Fri, 05 Sep 2008) $
*/
public class MenuItemBean 
	implements MenuItemAction 
{

	private String menuId;
	private String submenuId;
	private String icon;
	private String onClick;
	private String style;
	private String title;
	private String tooltip;
	private boolean enabled;
	private boolean visible;
	private MenuType type;

	/**
		Create a new empty menu item action.
	*/
	public MenuItemBean()
	{
		type = MenuType.MENUITEM;
		enabled = true;
		visible = true;
	}
	
	/**
		Create a new initialised menu item action
	*/
	public MenuItemBean(MenuType type, String menuId, String submenuId,
		String title, String onClick, String tooltip, String icon, String style,
		boolean enabled, boolean visible)
	{
		this.type = type;
		this.menuId = menuId;
		this.submenuId = submenuId;
		this.title = title;
		this.onClick = onClick;
		this.tooltip = tooltip;
		this.icon = icon;
		this.style = style;
		this.enabled = enabled;
		this.visible = visible;
	}
	
	/*
		From the MenuItemAction interface
		--------------------------------
	*/
	@Override
	public String getIcon() 
	{
		return icon;
	}

	@Override
	public String getMenuId()
	{
		return menuId;
	}
	
	@Override
	public String getOnClick() 
	{
		return onClick;
	}

	@Override
	public String getStyle() 
	{
		return style;
	}
	
	@Override
	public String getSubmenuId()
	{
		return submenuId;
	}

	@Override
	public String getTitle()
	{
		return title;
	}

	@Override
	public String getTooltip() 
	{
		return tooltip;
	}

	@Override
	public MenuType getType() 
	{
		return type;
	}

	@Override
	public boolean isEnabled() 
	{
		return enabled;
	}

	@Override
	public boolean isVisible() 
	{
		return visible;
	}
	// ------------------------------------

	public void setIcon(String icon) 
	{
		this.icon = icon;
	}

	public void setMenuId(String menuId)
	{
		this.menuId = menuId;
	}
	
	public void setOnClick(String onClick)
	{
		this.onClick = onClick;
	}

	public void setStyle(String style) 
	{
		this.style = style;
	}
	
	public void setSubmenuId(String submenuId)
	{
		this.submenuId = submenuId;
	}

	public void setTitle(String title) 
	{
		this.title = title;
	}

	public void setTooltip(String tooltip) 
	{
		this.tooltip = tooltip;
	}

	public void setEnabled(boolean enabled) 
	{
		this.enabled = enabled;
	}
	
	public void setVisible(boolean visible) 
	{
		this.visible = visible;
	}

	public void setType(MenuType type) 
	{
		this.type = type;
	}

}
