/**
	$Id: ExtendedPropertyFormatter.java 4478 2008-09-05 15:15:54Z jari $

	Copyright (C) 2006 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.clients.web.formatter;

import net.sf.basedb.core.ExtendedProperty;
import net.sf.basedb.util.formatter.Formatter;

/**
	Formatter for extended property values. The actual property value is formatted
	by the supplied formatter. This formatter then checks if the extended
	property should be linked by calling the {@link ExtendedProperty#getUrl(Object)}
	method.

	@author nicklas
	@version 2.0
	@base.modified $Date: 2008-09-05 17:15:54 +0200 (Fri, 05 Sep 2008) $
*/
public class ExtendedPropertyFormatter<T>
	implements Formatter<T>
{

	private ExtendedProperty ep;
	private Formatter<T> valueFormatter;
	
	/**
		Create an extended property formatter.
		@param ep The extended property
		@param valueFormatter Formatter to use for the values
	 */
	public ExtendedPropertyFormatter(ExtendedProperty ep, Formatter<T> valueFormatter)
	{
		this.ep = ep;
		this.valueFormatter = valueFormatter;;
	}
	
	/*
		From the Formatter interface
		-------------------------------------------
	*/
	public String format(T value)
	{
		String link = ep.getUrl(value);
		String formattedValue = valueFormatter.format(value);
		if (link != null)
		{
			formattedValue = "<a href=\""+link+"\" target=\"_blank\">"+value+"</a>";
		}
		return formattedValue;
	}
	public T parseString(String value)
	{
		return valueFormatter.parseString(value);
	}
	// -------------------------------------------

}
