/**
	$Id: SpotImageFormatter.java 4478 2008-09-05 15:15:54Z jari $

	Copyright (C) 2006 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.clients.web.formatter;

import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.RawBioAssay;
import net.sf.basedb.core.RawDataType;
import net.sf.basedb.core.RawDataUtil;
import net.sf.basedb.core.data.RawData;
import net.sf.basedb.util.formatter.Formatter;

/**

	@author nicklas
	@version 2.0
	@base.modified $Date: 2008-09-05 17:15:54 +0200 (Fri, 05 Sep 2008) $
*/
public class SpotImageFormatter
	implements Formatter<Integer>
{

	private final DbControl dc;
	private final RawDataType rawDataType;
	private final String ID;
	private final String root;
	
	public SpotImageFormatter(DbControl dc, RawDataType rawDataType, String root)
	{
		this.dc = dc;
		this.rawDataType = rawDataType;
		this.root = root;
		this.ID = dc.getSessionControl().getId();
	}
	
	/*
		From the Formatter interface
		-------------------------------------------
	*/
	public String format(Integer value)
	{
		RawData rawData = rawDataType.getRawDataById(dc, value);
		RawBioAssay rawBioAssay = RawDataUtil.getRawBioAssay(dc, rawData);
		
		StringBuilder sb = new StringBuilder();
		if (rawBioAssay.hasSpotImages())
		{
			sb.append("<img src=\"").append(root).append("/views/rawbioassays/rawdata/spotimage/");
			sb.append(ID).append("/").append(rawBioAssay.getId()).append("/");
			sb.append(rawData.getId()).append("/spot").append(rawData.getPosition()).append(".png");
			sb.append("\">");
		}
		else
		{
			sb.append("<i>- none -</i>");
		}
		return sb.toString();
	}
	/**
		Not supported.
		@since 2.2
	*/
	public Integer parseString(String value)
	{
		throw new UnsupportedOperationException("parseString");
	}
	// -------------------------------------------

}
