/**
	$Id: DataQueryWrapper.java 4478 2008-09-05 15:15:54Z jari $

	Copyright (C) 2006 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.clients.web.plugins;

import net.sf.basedb.core.DataQuery;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.Metadata;
import net.sf.basedb.core.data.BasicData;
import net.sf.basedb.core.query.ResultIterator;

/**
 	A query wrapper for {@link DataQuery}:s.
 	
	@author nicklas
	@version 2.2
	@base.modified $Date: 2008-09-05 17:15:54 +0200 (Fri, 05 Sep 2008) $
*/
public class DataQueryWrapper
	implements QueryWrapper<BasicData>
{

	private final DataQuery<BasicData> query;
	private final DbControl dc;
	private final Item itemType;
	private final Metadata metadata;

	/**
		Create a new query wrapper.
		@param dc The DbControl used to execute the query
		@param query The query
		@param itemType Type of the items returned by the query
		@param entityName An optional entity name when retruning raw data
	*/
	public DataQueryWrapper(DbControl dc, DataQuery<BasicData> query, Item itemType, String entityName)
	{
		this.dc = dc;
		this.query = query;
		this.itemType = itemType;
		this.metadata = entityName == null ? 
			Metadata.getInstance(itemType.getDataClass()) : 
			Metadata.getInstance(itemType.getDataClass(), entityName);
	}

	/**
		Calls {@link DataQuery#iterate(DbControl)}
	*/
	public ResultIterator<BasicData> iterate()
	{
		return query.iterate(dc);
	}
	/**
		Calls {@link BasicData#getId()}
	*/
	public int getId(BasicData item)
	{
		return item.getId();
	}
	/**
		Uses a {@link Metadata} object to get the value of the specified property.
	*/
	@SuppressWarnings("unchecked")
	public Object getData(ExportedProperty exportedProperty, BasicData item)
	{
		return metadata.getPropertyValue(dc, item, exportedProperty.name);
	}

}
