/**
	$Id: DynamicQueryWrapper.java 4478 2008-09-05 15:15:54Z jari $

	Copyright (C) 2006 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.clients.web.plugins;

import java.sql.SQLException;

import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.DynamicQuery;
import net.sf.basedb.core.query.ResultIterator;
import net.sf.basedb.core.query.SqlResult;

/**
	A query wrapper for {@link DynamicQuery}:s.
	
	@author nicklas
	@version 2.2
	@base.modified $Date: 2008-09-05 17:15:54 +0200 (Fri, 05 Sep 2008) $
*/
public class DynamicQueryWrapper
	implements QueryWrapper<SqlResult>
{
	private final DynamicQuery query;
	private final DbControl dc;
	private int counter = 0;
	private int colIndex;

	/**
		Create a new query wrapper.
		@param dc The DbControl used to execute the query
		@param query The query
	*/
	public DynamicQueryWrapper(DbControl dc, DynamicQuery query)
	{
		this.dc = dc;
		this.query = query;
	}
	
	/**
		Calls {@link DynamicQuery#iterate(DbControl)}
	*/
	public ResultIterator<SqlResult> iterate()
	{
		return query.iterate(dc);
	}

	/**
		Keeps a counter which is incremented at each call to this method.
		Starts at 0.
	*/
	public int getId(SqlResult item)
	{
		colIndex = 0;
		return counter++;
	}

	/**
		Calls {@link SqlResult#getObject(int)} and increases the column index
		for each call. Expects this method to be called in the order the properties 
		are selected.
	*/
	public Object getData(ExportedProperty exportedProperty, SqlResult item)
		throws SQLException
	{
		colIndex++;
		return item.getObject(colIndex);
	}

}
