/*
	$Id: CompileAll.java 4478 2008-09-05 15:15:54Z jari $

	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.clients.web.servlet;

import javax.servlet.RequestDispatcher;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletRequestWrapper;
import javax.servlet.http.HttpServletResponse;

import net.sf.basedb.core.Application;

import java.io.IOException;
import java.io.PrintWriter;

import java.util.Comparator;
import java.util.Set;
import java.util.TreeSet;

/**
	This servlet will scan for all files ending with .jsp and issue
	a request for them with the parameter <code>jsp_precompile=true</code>
	therby compiling all JSP files into servlets.
	<p>
	This code is based on code written by Ral Gagnon:
	<a href="http://www.rgagnon.com/javadetails/java-0414.html">http://www.rgagnon.com/javadetails/java-0414.html</a>

	@author nicklas
	@version 2.0
	@base.modified $Date: 2008-09-05 17:15:54 +0200 (Fri, 05 Sep 2008) $
*/
public final class CompileAll
	extends HttpServlet
{

	/**
		
	*/
	private static final long serialVersionUID = 5908370094706141413L;
	private static final Comparator pathComparator = new Comparator<String>()
	{
		/**
			If both are files or both are directories use regular 
			String comparison, otherwise put files before directories.
		*/
		public int compare(String s1, String s2)
		{
			boolean s1IsDirectory = s1.endsWith("/");
			boolean s2IsDirectory = s2.endsWith("/");
			if (s1IsDirectory == s2IsDirectory)
			{
				return s1.compareTo(s2);
			}
			else
			{
				return s1IsDirectory ? 1 : -1;
			}
		}
	};
	
	public void doGet(HttpServletRequest request, HttpServletResponse response)
		throws IOException, ServletException
	{
	
		// Wrap the original request object and set the query string to "jsp_precompile=true"
		HttpServletRequest wrappedRequest = new HttpServletRequestWrapper(request) 
		{
			public String getQueryString() 
			{
				return "jsp_precompile=true";
			}
		};

		response.setContentType("text/html");
		PrintWriter out = response.getWriter();
		out.print("<html>\n<head>\n<script language=\"JavaScript\">\n");
		out.print("function toggle(id)\n");
		out.print("{\n");
		out.print("var element = document.getElementById(id);\n");
		out.print("var currentDisplay = element.style.display;\n");
		out.print("element.style.display = currentDisplay == 'block' ? 'none' : 'block';\n");
		out.print("}\n");
		out.print("</script>\n</head>\n<body>\n");
		boolean success = false;
		try
		{
			out.print("Starting BASE ...");
			out.flush();
			Application.start();
			out.print(" done<hr>");
			out.flush();
			success = compile("/", wrappedRequest, response);
		}
		catch (Exception ex)
		{
			out.print(" failed<br>");
			out.print("<pre>");
			out.print(ex.getMessage());
			ex.printStackTrace(out);
			out.print("</pre>");
			out.flush();
		}
		out.print("<hr>");
		out.print(success ? "Compilation successful" : "Not all pages could be compiled");
		out.print("</body></html>\n");
		out.flush();
	}

	public void doPost(HttpServletRequest request, HttpServletResponse response)
		throws IOException, ServletException
	{
		doGet(request, response);
	}
	
	/**
		Compile all files found in dir, recursively entering subdirectories.
		
		@param dir The starting directory
		@param request
		@param response
		@throws IOException
	 */
	@SuppressWarnings("unchecked")
	private boolean compile(String dir, HttpServletRequest request, HttpServletResponse response)
		throws IOException
	{
		PrintWriter out = response.getWriter();
		boolean ok = true;
		
		// Get all files in the "dir" directory
		Set<String> all = new TreeSet<String>(pathComparator);
		all.addAll(getServletContext().getResourcePaths(dir));
		
		for (String path : all)
		{
			if (path.endsWith(".jsp"))
			{
				out.print("Compiling: " + path + " ... ");
				out.flush();
				RequestDispatcher rd = getServletContext().getRequestDispatcher(path);
				if (rd == null)
				{
					out.print(" not found<br>");
					out.flush();
				}
				else
				{
					try
					{
						rd.include(request, response);
						out.print(" done<br>");
						out.flush();
					}
					catch (Exception ex)
					{
						ok = false;
						int exId = ex.hashCode();
						out.print(" <a href=\"javascript:void(0)\" onclick=\"toggle('"+exId+"');\" title=\"Show/hide stacktrace\">failed</a><br>");
						out.print("<div id=\""+exId+"\" style=\"display: none\"><pre>");
						out.print(ex.getMessage()+"\n");
						ex.printStackTrace(out);
						out.print("</pre></div>");
					}
				}
			}
			else if (path.endsWith("/"))
			{
				ok &= compile(path, request, response);
			}
		}
		return ok;
	}
}
