/*
	$Id: Render.java 4478 2008-09-05 15:15:54Z jari $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.clients.web.taglib.extensions;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.TagSupport;

import net.sf.basedb.clients.web.extensions.JspContext;
import net.sf.basedb.util.extensions.ExtensionsInvoker;
import net.sf.basedb.util.extensions.Renderer;

/**
	Tag for rendering extensions.

	<p>
	<b>Syntax:</b>
	<pre>
   &lt;ext:render 
      extensions=...
      context=...
      item=...
      renderer=...
  /&gt;
</pre>
	<p>
	<table border=1 cellspacing=0 cellpadding=2>
	<tr>
		<td><b>Attribute</b></td>
		<td><b>Default value</b></td>
		<td><b>Required</b></td>
		<td><b>Description</b></td>
	</tr>
	<tr>
		<td>extensions</td>
		<td>-</td>
		<td>yes</td>
		<td>
			The {@link ExtensionsInvoker} with the extensions.
		</td>
	</tr>
	<tr>
		<td>context</td>
		<td>-</td>
		<td>no</td>
		<td>
			The current {@link JspContext}.
		</td>
	</tr>
	<tr>
		<td>item</td>
		<td>-</td>
		<td>no</td>
		<td>
			The current item. If not null and the context is also not null,
			the context will be updated with {@link JspContext#setCurrentItem(Object)} 
			before the rendering is started.
		</td>
	</tr>
	<tr>
		<td>renderer</td>
		<td>-</td>
		<td>no</td>
		<td>
			The renderer to use. If not null, the 
			{@link ExtensionsInvoker#render(net.sf.basedb.util.extensions.Renderer)}
			method is used, otherwise {@link ExtensionsInvoker#renderDefault()}.
		</td>
	</tr>
	</table>

	@author Nicklas
	@version 2.7
*/
public class Render 
	extends TagSupport
{

	private static final long serialVersionUID = -788121059344311519L;

	// The JSP context
	private JspContext context = null;

	// The extensions invoker
	private ExtensionsInvoker extensions = null;
	
	// The current item
	private Object item = null;
	
	// The renderer to use
	private Renderer renderer = null;
	
	public void setExtensions(ExtensionsInvoker extensions)
	{
		this.extensions = extensions;
	}
	
	public void setContext(JspContext context)
	{
		this.context = context;
	}
	
	public void setItem(Object item)
	{
		this.item = item;
	}
	
	public void setRenderer(Renderer renderer)
	{
		this.renderer = renderer;
	}

	@SuppressWarnings("unchecked")
	public int doStartTag() 
		throws JspException
	{
		if (item != null && context != null) context.setCurrentItem(item);
		if (renderer == null)
		{
			extensions.renderDefault();
		}
		else
		{
			extensions.render(renderer);
		}
		return SKIP_BODY;
	}
}


