/*
	$Id: Batcher.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.core;

/**
	The basic interface which all batchers must implement.


	@author Nicklas
	@version 2.0
	@see <a href="../../../../../development/overview/core/batchprocessing.html">Batch processing overview</a>
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
*/
public interface Batcher
{
	/**
		Get the current batch size of the batcher. The batch size indicates how
		many individual items a batcher is caching in memory before they are
		automatically flushed to the database.
		
		@return The current batch size, or 0 if automatic flushing is turned of
	*/
	public int getBatchSize();
	
	/**
		Sets the batch size. The batch size indicates how
		many individual items a batcher is caching in memory before they are
		automatically flushed to the database.
		
		@param batchSize Size of the batch, or 0 to disable automatic flushing
	*/
	public void setBatchSize(int batchSize);

	/**
		Check if the batcher has been closed or not. A closed batcher will
		not accept any more data for insert, update or delete. Note! A batcher
		that uses an underlying {@link DbControl} to connect to the database
		will automatically be closed when the <code>DbControl</code> is closed. 
		
		@return TRUE if the batcher is closed, FALSE otherwise
		@see #close()
	*/
	public boolean isClosed();

	/**
		Flush the batcher and send all remaining items in memory to the
		database.
		@throws BaseException If there is an error
		@see #close()
	*/
	public void flush()
		throws BaseException;

	/**
		Flush the batcher and close it. A closed batcher will
		not accept any more data for insert, update or delete. Note! A batcher
		that uses an underlying {@link DbControl} to connect to the database
		will automatically be closed when the <code>DbControl</code> is closed. 
		
		@throws BaseException If there is an error
		@see #isClosed()
		@see #flush()
	*/
	public void close()
		throws BaseException;

}
