/*
	$Id: QuotaException.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) 2005 Johan Enell, Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen
	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.core;

import net.sf.basedb.core.data.QuotaTypeData;
import net.sf.basedb.util.Values;

/**
	This exception is thrown to signal that the user's quota has been exceeded
	and that an {@link DiskConsumable} item cannot be saved.

	@author Nicklas
	@version 2.0
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
*/
public class QuotaException
	extends InvalidDataException
{
	/**
		
	*/
	private static final long serialVersionUID = 4279045703338054258L;

	/**
		Create a new <code>QuotaException</code> object.
	*/
	public QuotaException()
	{
		super("Quota exceeded.");
	}

	/**
		Create a new <code>QuotaException</code> object.

		@param location The location
		@param quotaType The quota type
		@param maxBytes The maximum allowed number of bytes
		@param currentBytes The currently used number of bytes
	*/
	public QuotaException(Location location, QuotaTypeData quotaType, long maxBytes, long currentBytes)
	{
		super(maxBytes == Quota.UNDEFINED ? 
			"Quota undefined for type " + quotaType.getName() + " on location " + location + "." 
			: "Quota exceeded. The currently used quota ("+Values.formatBytes(currentBytes)+
			" bytes) exceededs the quota limit ("+Values.formatBytes(maxBytes)+" bytes)"
			+" on location "+location+" for quota type "+quotaType.getName()+".");
	}
}


