/*
	$Id: RawDataUtil.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.core;

import net.sf.basedb.core.data.RawData;
import net.sf.basedb.core.data.RawBioAssayData;

import java.lang.reflect.Method;

/**
	Utiltity class for getting and setting non-public properties of
	raw data objects.

	@author Nicklas
	@version 2.0
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
*/
public class RawDataUtil
{

	/**
		Method for calling {@link RawData#getRawBioAssay()}
	*/
	private static Method getRawBioAssayData;
	
	private static boolean isInitialised = false;

	/**
		Initialise the class.
	*/
	static synchronized void init()
		throws BaseException
	{
		if (isInitialised) return;
		try
		{
			getRawBioAssayData = RawData.class.getDeclaredMethod("getRawBioAssay");
			getRawBioAssayData.setAccessible(true);
		}
		catch (NoSuchMethodException ex)
		{
			throw new BaseException(ex);
		}
		isInitialised = true;
	}
	
	/**
		Unload all settings.
	*/
	static synchronized void unload()
	{
		isInitialised = false;
		getRawBioAssayData = null;
	}

	/**
		Get the raw bioassay the raw data belongs to.
		@param dc The <code>DbControl</code> object used for permission
			checking and database access
		@param rawData The raw data object
		@return A <code>RawBioAssay</code> object
		@throws PermissionDeniedException If the logged in user doesn't have
			read permission
		@throws BaseException If there is another error
	*/
	public static RawBioAssay getRawBioAssay(DbControl dc, RawData rawData)
		throws PermissionDeniedException, BaseException
	{
		RawBioAssayData rba = null;
		try
		{
			rba = (RawBioAssayData)getRawBioAssayData.invoke(rawData);
		}
		catch (Exception ex)
		{
			throw new BaseException(ex);
		}
		return dc.getItem(RawBioAssay.class, rba);
	}

}
