/*
	$Id: ScrollIterator.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.core;

import org.hibernate.ScrollableResults;
import java.util.Iterator;
import java.util.NoSuchElementException;

/**
	This is an iterator view of the result from a query
	using the Hibernate <code>ScrollableResults</code> object.
	
	@author Nicklas
	@version 2.0
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
*/
class ScrollIterator<E>
	implements Iterator<E>
{

	private final Class<E> dataClass;
	private final ScrollableResults results;
	private boolean isClosed;
	private E nextElement;

	ScrollIterator(Class<E> dataClass, ScrollableResults results)
	{
		assert dataClass != null : "dataClass == null";
		assert results != null : "results == null";
		this.dataClass = dataClass;
		this.results = results;
		this.isClosed = false;
		this.nextElement = null;
	}
	
	/*
		From the Iterator interface
		---------------------------
	*/
	public boolean hasNext()
	{
		if (nextElement == null)
		{
			if (!isClosed()) 
			{
				if (results.next())
				{
					nextElement = dataClass.cast(results.get(0));
				}
				else
				{
					close();
				}
			}
		}
		return nextElement != null;
	}
	public E next()
	{
		if (!hasNext()) throw new NoSuchElementException();
		E temp = nextElement;
		nextElement = null;
		return temp;
	}
	/**
		Not supported.
		@throws UnsupportedOperationException Always
	*/
	public void remove()
	{
		throw new UnsupportedOperationException();
	}
	// ---------------------------------

	/**
		Close the results and return resources.
	*/
	public void close()
	{
		if (!isClosed()) results.close();
		isClosed = true;
	}
	/**
		Check if the results has been closed.
	*/
	public boolean isClosed()
	{
		return isClosed;
	}
	
}
