/*
	$Id: SimpleAuthenticator.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) 2005 Samuel Andersson, Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.core.authentication;


/**
	This class is an example implementation of the {@link Authenticator}
	interface that authenticates everyone as long as the login is the same
	as the password. This class should of course never be used in a real
	environment.

	@author Nicklas
	@version 2.0
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
*/
public class SimpleAuthenticator
	implements Authenticator
{
	/**
		Create a new <code>SimpleAuthenticator</code> object.
	*/
	public SimpleAuthenticator()
	{
		super();
	}

	/**
		There is nothing to initialise.
	*/
	public void init(String settings)
		throws AuthenticationException
	{}

	/**
		Always return FALSE.
	*/
	public boolean supportsExtraInformation()
	{
		return false;
	}

	/**
		If login and password are equal the user is authenticated, otherwise an
		{@link InvalidPasswordException} is thrown.
	*/
	public AuthenticationInformation authenticate(String login, String password)
		throws UnknownLoginException, InvalidPasswordException, AuthenticationException
	{
		if (login == null || !login.equals(password))
		{
			throw new InvalidPasswordException(login, "Password must be equal to login");
		}
		return new AuthenticationInformation(login, login);
	}



}
