/*
  $Id: AnnotationData.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

  BASE is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 3
  of the License, or (at your option) any later version.

  BASE is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330,
  Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.core.data;

import java.util.Set;

/**
	This class holds information about an annotation
	
	@author Nicklas
	@version 2.0
	@see net.sf.basedb.core.Annotation
	@see <a href="../../../../../../html/developerdoc/api/data_api.annotations.html">Annotations overview</a>
	@hibernate.class table="`Annotations`" lazy="false"
 */
public class AnnotationData
	extends BasicData
{
	public AnnotationData()
	{}

	private AnnotationSetData annotationSet;
	/**
		Get the annotation set this annotation primarily belongs to.
		@hibernate.many-to-one outer-join="false" update="false"
		@hibernate.column name="`annotationset_id`" not-null="true" unique-key="uniquetype"
	*/
	public AnnotationSetData getAnnotationSet()
	{
		return annotationSet;
	}
	public void setAnnotationSet(AnnotationSetData annotationSet)
	{
		this.annotationSet = annotationSet;
	}

	private AnnotationTypeData annotationType;
	/**
		Get the type of this annotation.
		@hibernate.many-to-one outer-join="false" update="false"
		@hibernate.column name="`annotationtype_id`" not-null="true" unique-key="uniquetype"
	*/
	public AnnotationTypeData getAnnotationType()
	{
		return annotationType;
	}
	public void setAnnotationType(AnnotationTypeData annotationType)
	{
		this.annotationType = annotationType;
	}
	
	private ParameterValueData<?> values;
	/**
		The values of this annotation.
		@hibernate.many-to-one column="`value_id`" not-null="true" update="false" cascade="all" unique="true"
	*/
	public ParameterValueData<?> getValues()
	{
		return values;
	}
	public void setValues(ParameterValueData<?> values)
	{
		this.values = values;
	}
	
	private Set<AnnotationSetData> inheritingSets;
	/**
		The annotation sets which inherit this annotation. This is the inverse end.
		@see AnnotationSetData#getInherited()
		@hibernate.set table="`InheritedAnnotations`" lazy="true"
		@hibernate.collection-key column="`annotation_id`"
		@hibernate.collection-many-to-many column="`annotationset_id`" class="net.sf.basedb.core.data.AnnotationSetData"
	*/
	Set<AnnotationSetData> getInheritingSets()
	{
		return inheritingSets;
	}
	void setInheritingSets(Set<AnnotationSetData> inheritingSets)
	{
		this.inheritingSets = inheritingSets;
	}


}
