/*
	$Id: FileTypeIndex.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.core.data;

import java.io.Serializable;

/**
	This is a helper class for the {@link PlatformData} item to
	help with the Hibernate mapping to {@link PlatformFileTypeData}
	object.

	@author Nicklas
	@version 2.5
	@see <a href="../../../../../../html/developerdoc/api/data_api.platforms.html">Data API - Experimental platforms</a>
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
*/
public class FileTypeIndex
	implements Serializable
{
	/**
		
	*/
	private static final long serialVersionUID = 5213031457561980770L;
	FileTypeIndex()
	{}
	
	public FileTypeIndex(PlatformVariantData variant, DataFileTypeData fileType)
	{
		this.fileType = fileType;
		this.variant = variant;
	}

	private transient PlatformVariantData variant;
	/**
	 	@hibernate.many-to-one column="`variant_id`" outer-join="false"
	*/
	public PlatformVariantData getVariant()
	{
		return variant;
	}
	void setVariant(PlatformVariantData variant)
	{
		this.variant = variant;
	}

	private transient DataFileTypeData fileType;
	/**
	 	@hibernate.many-to-one column="`datafiletype_id`" outer-join="false"
	*/
	public DataFileTypeData getFileType()
	{
		return fileType;
	}
	void setFileType(DataFileTypeData fileType)
	{
		this.fileType = fileType;
	}

	/**
		Check if this object is equal to another <code>FileTypeIndex</code>
		object. They are equal if both have the same fileType and variant id.
	*/
	public boolean equals(Object o)
	{
		if (this == o) return true;
		if ((o == null) || (getClass() != o.getClass()))
		{
			return false;
		}
		FileTypeIndex other = (FileTypeIndex) o;
		// Same file type
		if (this.fileType.equals(other.fileType))
		{
			// Same or no variant
			if ((this.variant == other.variant) || 
				(this.variant != null && this.variant.equals(other.variant))) return true;
		}
		return false;
	}

	/**
		Calculate the hash code for the object.
	*/
	public int hashCode()
	{
		return (int) (53 * fileType.getId() + 17 * (variant == null ? 0 : variant.getId()));
	}
}
