/*
	$Id: PlateCoordinate.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.core.data;

import java.io.Serializable;

/**
	This class defines a plate coordinate, which is row and column
	on a plate. A <code>PlateCoordinate</code> object
	is used by a {@link PlateData} to specify the coordinates of wells. 
	The <code>Serializable</code> interface must be implemented
	because Hibernate requires it if we use it as a map key.

	@author Nicklas
	@version 2.0
	@see PlateData
	@see <a href="../../../../../../html/developerdoc/api/data_api.plates.html">Array LIMS - Plates overview</a>
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
*/
public class PlateCoordinate
	implements Serializable
{
	/**
		
	*/
	private static final long serialVersionUID = -7499789811217707905L;
	/**
		The row number.
	*/
	private int row;
	/**
		The column number.
	*/
	private int column;

	/**
		Create new object. Used by Hibernate only.
	*/
	PlateCoordinate()
	{}

	/**
		Create new <code>PlateCoordinate</code> with the specified row and column.
	*/
	public PlateCoordinate(int row, int column)
	{
		this.row = row;
		this.column = column;
	}

	/*
		From the Object class
		-------------------------------------------
	*/
	/**
		Check if this object is equal to another <code>PlateCoordinateData</code>
		object. They are equal if both have the same plate, row and column.
	*/
	public final boolean equals(Object o)
	{
		if (this == o) return true;
		if ((o == null) || (getClass() != o.getClass())) return false;
		PlateCoordinate pc = (PlateCoordinate)o;
		return
			(this.row == pc.row) &&
			(this.column == pc.column);
	}
	/**
		Calculate the hash code for the object.
	*/
	public final int hashCode()
	{
		return (int)(53*row + 17*column);
	}
	/**
		Get the coordinate as [row, column]
	*/
	public final String toString()
	{
		return "["+row+", "+column+"]";
	}
	// -------------------------------------------

	/**
		Get the row number.
		@hibernate.property column="`row`" type="int"
	*/
	public int getRow()
	{
		return row;
	}
	/**
		Set the row number.
	*/
	void setRow(int row)
	{
		this.row = row;
	}

	/**
		Get the column number.
		@hibernate.property column="`column`" type="int"
	*/
	public int getColumn()
	{
		return column;
	}
	/**
		Set the location.
	*/
	void setColumn(int column)
	{
		this.column = column;
	}

}

