/*
	$Id: PlateMappingData.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.core.data;

import java.util.Map;
import java.util.HashMap;

/**
	This class holds information about a plate mapping.

	@author Nicklas
	@version 2.0
	@see net.sf.basedb.core.PlateMapping
	@see <a href="v">Array LIMS - Plates overview</a>
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
	@hibernate.class table="`PlateMappings`" lazy="true"
*/
public class PlateMappingData
	extends CommonData
{

	public PlateMappingData()
	{}
	
	private PlateGeometryData sourceGeometry;
	/**
		The required geometry of source plates in this mapping.
		@hibernate.many-to-one column="`sourcegeometry_id`" not-null="true" outer-join="false" update="false"
	*/
	public PlateGeometryData getSourceGeometry()
	{
		return sourceGeometry;
	}
	public void setSourceGeometry(PlateGeometryData sourceGeometry)
	{
		this.sourceGeometry = sourceGeometry;
	}

	private PlateGeometryData destinationGeometry;
	/**
		The required geometry of destination plates in this mapping.
		@hibernate.many-to-one column="`destinationgeometry_id`" not-null="true" outer-join="false" update="false"
	*/
	public PlateGeometryData getDestinationGeometry()
	{
		return destinationGeometry;
	}
	public void setDestinationGeometry(PlateGeometryData destinationGeometry)
	{
		this.destinationGeometry = destinationGeometry;
	}
	
	private int sourceCount;
	/**
		The required number of source plates.
		@hibernate.property column="`source_count`" type="int" not-null="true" update="false"
	*/
	public int getSourceCount()
	{
		return sourceCount;
	}
	public void setSourceCount(int sourceCount)
	{
		this.sourceCount = sourceCount;
	}

	private int destinationCount;
	/**
		The produced number of destination plates.
		@hibernate.property column="`destination_count`" type="int" not-null="true" update="false"
	*/
	public int getDestinationCount()
	{
		return destinationCount;
	}
	public void setDestinationCount(int destinationCount)
	{
		this.destinationCount = destinationCount;
	}
	
	/**
		The maximum length of the image name that can be stored in the database.
		@see #setImage(String)
	*/
	public static final int MAX_IMAGE_LENGTH = 255;
	private String image;
	/**
		An optional name of a small image with a graphical representation of the mapping.
		@hibernate.property column="`image`" type="string" size="255" not-null="false"
	*/
	public String getImage()
	{
		return image;
	}
	public void setImage(String image)
	{
		this.image = image;
	}
	
	private Map<MappingCoordinate, MappingCoordinate> details;
	/**
		For each destination coordinate we can get the source coordinate.
		The mapping for this element is specified in 
		<code>hibernate-properties-PlateMappingData.xml</code>
		because we must supply different column names for the properties of
		{@link MappingCoordinate} objectes depending on if it is a destination 
		coordinate (map key) or source coordinate (map value), which is not 
		supported by XDoclet.

		The structure of the mapping is:
		hibernate.map table="`PlateMappingDetails`" lazy="true" cascade="delete"
		hibernate.collection-key column="`platemapping_id`" type="int"
		hibernate.collection-composite-index class="net.sf.basedb.core.data.MappingCoordinate"
		hibernate.collection-composite-element class="net.sf.basedb.core.data.MappingCoordinate"
	*/
	public Map<MappingCoordinate, MappingCoordinate> getDetails()
	{
		if (details == null) details = new HashMap<MappingCoordinate, MappingCoordinate>();
		return details;
	}
	void setDetails(Map<MappingCoordinate, MappingCoordinate> details)
	{
		this.details = details;
	}

}
