/*
  $Id: ReporterListData.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

  BASE is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 3
  of the License, or (at your option) any later version.

  BASE is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330,
  Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.core.data;

import java.util.Map;
import java.util.HashMap;

/**
	This class holds information about a reporter list.
 
	@author Nicklas
	@version 2.0
	@see net.sf.basedb.core.ReporterList
	@see <a href="../../../../../../html/developerdoc/api/data_api.reporters.html">Reporter overview</a>
	@hibernate.class table="`ReporterLists`" lazy="true"
*/
public class ReporterListData
	extends CommonData
{

	/**
		The maximum length of the external ID that can be stored in the database.
		@see #setExternalId(String)
	*/
	public static final int MAX_EXTERNAL_ID_LENGTH = 255;
	private String externalId;
	/**
		Get the external id for this <code>ReporterList</code> item.
		@hibernate.property column="`external_id`" type="string" length="255" not-null="false"
	*/
	public String getExternalId()
	{
		return externalId;
	}
	public void setExternalId(String externalId)
	{
		this.externalId = externalId;
	}
	
	private ExperimentData experiment;
	/**
		Get the experiment this reporter list belongs to.
		@hibernate.many-to-one column="`experiment_id`" not-null="false" outer-join="false"
	*/
	public ExperimentData getExperiment()
	{
		return experiment;
	}
	public void setExperiment(ExperimentData experiment)
	{
		this.experiment = experiment;
	}
	private int size;
	/**
		The number of reporter in the list.
		@hibernate.property column="`size`" type="int" not-null="true"
		@since 2.8
	*/
	public int getSize()
	{
		return size;
	}
	public void setSize(int size)
	{
		this.size = size;
	}

	private Map<Integer, ReporterListScoreData> reporters;
	/**
		The list of reporters and their scores.
		@hibernate.map lazy="true" inverse="true" cascade="all-delete-orphan"
		@hibernate.collection-key column="`reporterlist_id`"
		@hibernate.collection-index column="`reporter_id`" type="int"
		@hibernate.collection-one-to-many class="net.sf.basedb.core.data.ReporterListScoreData"
	*/
	public Map<Integer, ReporterListScoreData> getReporterListScores()
	{
		if (reporters == null)
		{
			reporters = new HashMap<Integer, ReporterListScoreData>();
		}
		return reporters;
	}
	void setReporterListScores(Map<Integer, ReporterListScoreData> reporters)
	{
		this.reporters = reporters;
	}

	
}
