/*
	$Id: RoleKeyData.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) 2005 Samuel Andersson, Jari Hakkinen, Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.core.data;

import java.util.Map;
import java.util.HashMap;

/**
	This class holds information access privileges for roles.

	@author Nicklas
	@version 2.0
	@see net.sf.basedb.core.RoleKey
	@see <a href="../../../../../../html/developerdoc/api/data_api.authentication.html">Authentication overview</a>
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
	@hibernate.subclass discriminator-value="3"
*/
public class RoleKeyData
	extends KeyData
{

	public RoleKeyData()
	{}

	private String name;
	/**
		Get the name of the role key.
		@hibernate.property type="string"
		@hibernate.column name="`name`" type="string" length="255" not-null="false" index="name_idx"
	*/
	public String getName()
	{
		return name;
	}
	public void setName(String name)
	{
		this.name = name;
	}
	private String description;
	/**
		Get the description for the item.
		@return A <code>String</code> with a description of the item
		@hibernate.property column="`description`" type="text" not-null="false"
	*/
	public String getDescription()
	{
		return description;
	}
	public void setDescription(String description)
	{
		this.description = description;
	}

	private int itemType;
	/**
		Get the code for the type of items this role key applies to. See the
		{@link net.sf.basedb.core.Item} class for a list of constants.
		@hibernate.property column="`item_type`" type="int" not-null="false" update="false"
	*/
	public int getItemType()
	{
		return itemType;
	}
	public void setItemType(int itemType)
	{
		this.itemType = itemType;
	}

	private Map<RoleData,Integer> roles;
	/**
		Get the map that manages which roles that have permissions 
		for this key.

		@hibernate.map table="`RoleKeys`" lazy="true"
		@hibernate.collection-key column="`key_id`"
		@hibernate.index-many-to-many column="`role_id`" class="net.sf.basedb.core.data.RoleData"
		@hibernate.collection-element column="`permission`" type="int" not-null="true"
	*/
	public Map<RoleData,Integer> getRoles()
	{
		if (roles == null) roles = new HashMap<RoleData,Integer>();
		return roles;
	}
	void setRoles(Map<RoleData,Integer> roles)
	{
		this.roles = roles;
	}

}
