/*
	$Id: CdfFileHandler.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.core.filehandler;

import affymetrix.fusion.cdf.FusionCDFData;
import net.sf.basedb.core.ArrayDesign;
import net.sf.basedb.core.DataFileType;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.File;
import net.sf.basedb.core.FileSetMember;
import net.sf.basedb.core.InvalidDataException;
import net.sf.basedb.core.Platform;

/**
	File handler that can validate and extract metadata from
	Affymetrix CDF files.
	
	@author Nicklas
	@version 2.5
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
*/
public class CdfFileHandler
	extends AffymetrixFileHandler
	implements DataFileValidator, DataFileMetadataReader
{

	private FusionCDFData cdfData;
	
	public CdfFileHandler()
	{}
	
	/*
		From the DataFileValidator interface
		----------------------------------------
	*/
	/**
		Check that the array design is using the Affymetrix platform 
		({@link Platform#AFFYMETRIX}) and that the file selected
		as the CDF file really is a CDF file.
	*/
	public void validate(DbControl dc)
	{
		ArrayDesign design = (ArrayDesign)getItem();
		// Check platform
		if (!isAffymetrix(design))
		{
			throw new InvalidDataException("Array design '" + design.getName() + 
				"' is not using the Affymetrix platform");
		}
		
		// Get the member for CDF file type
		FileSetMember cdfMember = getMember(DataFileType.AFFYMETRIX_CDF);
		if (cdfMember != null)
		{
			// Verify CDF file
			File cdfFile = cdfMember.getFile();
			cdfData = loadCdfFile(cdfFile);
		}
	}
	
	/**
		Reads the number of probesets in the CDF file and stores it
		in {@link ArrayDesign#getNumFileFeatures()}.
	*/
	public void extractMetadata(DbControl dc)
	{
		if (cdfData == null)
		{
			FileSetMember cdfMember = getMember(DataFileType.AFFYMETRIX_CDF);
			if (cdfMember != null) cdfData = loadCdfFile(cdfMember.getFile());
		}
		if (cdfData != null)
		{
			ArrayDesign design = (ArrayDesign)getItem();
			design.setNumFileFeatures(cdfData.getHeader().getNumProbeSets());
		}
	}
	
	/**
		Reset {@link ArrayDesign#getNumFileFeatures()} to 0.
	*/
	public void resetMetadata(DbControl dc)
	{
		ArrayDesign design = (ArrayDesign)getItem();
		design.setNumFileFeatures(0);
	}
	// -------------------------------------------

}
