/**
	$Id: ExportOutputStream.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) 2006, 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.core.plugin;

import java.io.IOException;
import java.io.OutputStream;

/**
	An output stream used for immediate download from export plugins. The plugin
	must implement the {@link ImmediateDownloadExporter} interface. If the plugin
	wants to set mime type, content length and filename it must do so before it
	starts writing data to the output stream.
	<p>
	This class is intended to be subclassed by implementations more 
	suitable for the type of download. For example the {@link 
	net.sf.basedb.clients.web.util.ServletExportOutputStream} implements direct download
	from web clients and the {@link net.sf.basedb.util.FileExportOutputStream} implements
	direct download to the BASE file system.

	@author nicklas
	@version 2.2
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
*/
public class ExportOutputStream
	extends OutputStream
{

	private final OutputStream out;
	
	/**
		Create a new export stream. 
		@param out The underlying stream to write the exported data to
		@throws NullPointerException If out is null
	*/
	public ExportOutputStream(OutputStream out)
	{
		if (out == null) throw new NullPointerException("out");
		this.out = out;
	}
	
	/*
	 	From the OutputStream class
	 	---------------------------
	*/
	@Override
	public void close() 
		throws IOException
	{
		out.close();
	}
	@Override
	public void flush() 
		throws IOException
	{
		out.flush();
	}
	@Override
	public void write(byte[] b, int off, int len) 
		throws IOException
	{
		out.write(b, off, len);
	}
	@Override
	public void write(byte[] b) 
		throws IOException
	{
		out.write(b);
	}
	@Override
	public void write(int b) 
		throws IOException
	{
		out.write(b);
	}
	// -------------------------------------------

	/**
		Do nothing. Should be overridden by subclasses.
	*/
	public void setContentLength(long contentLength)
	{}
	
	/**
		Do nothing. Should be overridden by subclasses.
	*/
	public void setMimeType(String mimeType)
	{}
	
	/**
		Do nothing. Should be overridden by subclasses.
	*/
	public void setFilename(String name)
	{}
	
}
