/*
	$Id: GuiContext.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg
	
	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.core.plugin;

import net.sf.basedb.core.Item;

import java.util.Map;
import java.util.HashMap;

/**
	

	@author Nicklas
	@version 2.0
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
*/
public class GuiContext
{
	private final Item item;
	private final Type type;
	private final String subContext;
	
	/**
	 	@param item Item the context holds.
	 	@param type Type of context, 
	 		shows if it is a list or single view.
		
	*/
	public GuiContext(Item item, Type type)
	{
		this.item = item;
		this.type = type;
		this.subContext = null;
	}
	
	public GuiContext(Item item, Type type, String subContext)
	{
		this.item = item;
		this.type = type;
		this.subContext = subContext;
	}

	public Item getItem()
	{
		return item;
	}

	public Type getType()
	{
		return type;
	}

	public String getSubContext()
	{
		return subContext;
	}
	
	public boolean equals(Object other)
	{
		if (this == other) return true;
		if (!(other instanceof GuiContext)) return false;
		GuiContext gc = (GuiContext)other;
		return gc.item == this.item && gc.type == this.type;
	}
	
	public String toString()
	{
		return item + (type == Type.ITEM ? " (item)" : " (list)");
	}

	public enum Type
	{
		ITEM(0),
		LIST(1);
		/**
			Maps the integer that is stored in the database.
		*/
		private static final Map<Integer, Type> valueMapping = new HashMap<Integer, Type>();
		
		static
		{
			for(Type type : Type.values())
			{
				Type t = valueMapping.put(type.getValue(), type);
				assert t == null : "Another GuiContext.Type with the value "+type.getValue()+" already exists";
			}
		}
		
		/**
			Get the <code>GuiContext.Type</code> object when you know the integer code.
			
			@param value The integer value.
			@return The type object or null if the value is unknown
		*/
		public static Type fromValue(int value)
		{
			Type type = valueMapping.get(value);
			assert type != null : "type == null for value "+value;
			return type;
		}

		private final int value;
		private Type(int value)
		{
			this.value = value;
		}
		
		/**
			Get the integer value that is used when storing a <code>GuiContext.Type</code> 
			to the database.
			@return The intger value of this type 
		*/
		public int getValue()
		{
			return value;
		}
	}
}
