/*
	$Id: Expression.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) 2005 Samuel Andersson, Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.core.query;

/**
	An expression query element. This is a query element that can appear 
	almost anywhere in a query. For example a {@link Select} is usually 
	an expression together with an alias, a {@link Restriction} is
	a comparison of two or more expressions, etc.
	<p>
	There are many types of expressions. The simplest ones just insert a 
	constant value: {@link Expressions#integer(int)} or 
	{@link Expressions#aFloat(float)}.
	<p>
	Other expressions create a reference to a property or database column:
	{@link Hql#property(String, String)}.
	<p>
	Expressions can also be combined to form other expressions:
	{@link Expressions#add(Expression, Expression)} or {@link 
	Expressions#divide(Expression, Expression)}
	<p>
	Thus it is possible to form a complex tree of expressions:
<pre class="code">
<b>// The log ratio of foreground intensities: LOG(ch1fg) / LOG(ch2fg)</b>
Expression logRatio = Expressions.divide(
  Expressions.log(Hql.property("ch1fg")),
  Expressions.log(Hql.property("ch2fg"))
);
</pre>
	
	<p>
	The expressions can then be used to create restrictions:
<pre class="code">
<b>// Return only data with log ratio greater than 2.0</b> 
Restrictions.gt(logRatio, Expressions.aFloat(2.0));
</pre>
	
	<p>
	The {@link Expressions}, {@link Hql}, {@link Annotations} 
	and {@link Annotations} are factory classes that create expressions.
	
	
	@author Nicklas
	@version 2.0
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
*/
public interface Expression
	extends QueryElement
{}
