/*
	$Id: ExpressionSelect.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.core.query;

import java.util.Arrays;
import java.util.Collection;

import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.HibernateUtil;
import net.sf.basedb.core.BaseException;

/**
	A selection query element that selects an expression, optionally
	giving it an alias.
<pre class="code">
SELECT <i>&lt;expression&gt;</i> AS <i>&lt;alias&gt;</i>
FROM ...
</pre>

	@author Nicklas
	@version 2.0
	@see Selects#expression(Expression, String)
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
*/
public class ExpressionSelect
	implements Select
{
	private final Expression e1;
	private final String alias;
	private final boolean useInDistinctCount;

	ExpressionSelect(Expression e1, String alias, boolean useInDistinctCount)
	{
		this.e1 = e1;
		this.alias = alias;
		this.useInDistinctCount = useInDistinctCount;
	}

	/*
		From the QueryElement interface
		-------------------------------------------
	*/
	public String toQl(Query query, DbControl dc)
		throws BaseException
	{
		String quotedAlias = alias;
		if (query.getQueryType() == QueryType.SQL)
		{
			quotedAlias = HibernateUtil.quote(alias);
		}
		return e1.toQl(query, dc) + (alias == null || query.isCounting() ? "" : " AS "+quotedAlias);
	}
	/**
		@return The expressions to select
	*/
	public Collection<? extends QueryElement> getChildren()
	{
		return Arrays.asList(e1);
	}
	// -------------------------------------------
	/*
		From the Select interface
		-------------------------------------------
	*/
	public String getAlias()
	{
		return alias;
	}
	public Expression getExpression()
	{
		return e1;
	}
	public boolean useInDistinctCount()
	{
		return useInDistinctCount;
	}
	// -------------------------------------------
	/*
		From the Object class
		-------------------------------------------
	*/
	public String toString()
	{
		return e1.toString() + (alias == null ? "" : " AS "+alias);
	}
	// -------------------------------------------

}

