/*
	$Id: Join.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.core.query;

import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.DbControl;

/**
	A join query element. This is a query element that can appear in the
	<code>JOIN</code> part of a {@link Query}. Typically a join consists of
	join table and a join condition (SQL) or an association property (HQL).
	Examples:
	<pre class="code">
<b>// SQL</b>
SELECT hw.* 
FROM Hardware hw
JOIN HardwareTypes hwt ON hw.hardwaretype_id = hwt.id

<b>// HQL</b>
SELECT hw 
FROM HardwareData hw
JOIN hw.hardwareType hwt
</pre>

	<p>
	Joins for HQL can be created by the {@link Hql} factory class.

	@author Nicklas
	@version 2.0
	@see Hql
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
*/
public interface Join
	extends QueryElement
{
	public String getThetaJoin(Query query, DbControl dc)
		throws BaseException;

	public String getThetaJoinCondition(Query query, DbControl dc)
		throws BaseException;
}
