/**
	$Id: QuadraticMeanExpression.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C)

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/

package net.sf.basedb.core.query;

import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.DbControl;

import java.util.Collection;
import java.util.Collections;

/**
 	Calculates the quadratic mean of an expression 
  	@author Martin
  	@version 2.8
  	@see Aggregations#quadraticMean(Expression)
	@base.modified $Date: 2007-08-17 09:18:29 +0200 (Fri, 17 Aug 2007) $
 	
 */
public class QuadraticMeanExpression 
	implements Expression
{
	private final Expression e;
	
	private Expression sqrt;
	
	QuadraticMeanExpression(Expression e)
	{
		assert e != null : "e == null";
		this.e = e;
	}

	@Override
	/**
		@return The expression to find the quadratic mean for
	*/
	public Collection<? extends QueryElement> getChildren()
	{
		return Collections.singleton(e);
	}

	/*
		From the QueryElement interface
		-------------------------------------------
	*/
	@Override
	public String toQl(Query query, DbControl dc) 
		throws BaseException
	{
		return getSqrt().toQl(query, dc);
	}
	
	/*
	 	From the Object class
	 	-------------------------------
	 */
	public String toString()
	{
		return getSqrt().toString();
	}
	
	/*
	 	Gets the expression. Create a new if it not exists already.
	 */
	private Expression getSqrt()
	{
		if (sqrt == null)
		{
			sqrt = new SqrtExpression(new MeanExpression(new MultiplyExpression(e,e)));
		}
		return sqrt;
	}
}