/*
	$Id: RlikeRestriction.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) 2008 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.core.query;

import java.util.Arrays;
import java.util.Collection;

import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.HibernateUtil;

/**
	Checks if an expression matches a regular expression: 
	e RLIKE regexp (MySQL) or e ~ regexp (Postgres).

	@author Nicklas
	@version 2.8
	@see Restrictions#rlike(Expression, Expression)
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
*/
public class RlikeRestriction
	implements Restriction 
{
	private final Expression e;
	private final Expression regexp;

	RlikeRestriction(Expression e, Expression regexp)
	{
		assert e != null : "e == null";
		assert regexp != null : "regexp == null";
		this.e = e;
		this.regexp = regexp;
	}

	/*
		From the QueryElement interface
		-------------------------------------------
	*/
	public String toQl(Query query, DbControl dc)
		throws BaseException
	{
		if (query.getQueryType() == QueryType.HQL)
		{
			return "true = rlike(" + e.toQl(query, dc) + ", " + regexp.toQl(query, dc) + ")";
		}
		else
		{
			return HibernateUtil.getDbEngine().rlike(e.toQl(query, dc), regexp.toQl(query, dc));
		}
	}
	/**
		@return The expression
	*/
	public Collection<? extends QueryElement> getChildren()
	{
		return Arrays.asList(e, regexp);
	}
	// -------------------------------------------
	/*
		From the Object class
		-------------------------------------------
	*/
	public String toString()
	{
		return e + " ~ " + regexp;
	}
	// -------------------------------------------
}

