/*
	$Id: OwnableUtil.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) 2006 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.util;

import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.Ownable;
import net.sf.basedb.core.Permission;
import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.User;

import java.util.Set;

/**

	@author Nicklas
	@version 2.2
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
*/
public class OwnableUtil
{

	/**
		Change the owner of all specified items.
		@param dc The DbControl to use for database access
		@param itemType The type of items
		@param items A set with the ID:s of all items
		@param newOwner The new owner, or null to set the owner to the
			currently logged in user
		@return The number of items the owner could be changed on
		@throws BaseException If there is an error
	*/
	public static int setOwner(DbControl dc, Item itemType, Set<Integer> items, User newOwner)
		throws BaseException
	{
		int numOwned = 0;
		if (newOwner == null) newOwner = User.getById(dc, dc.getSessionControl().getLoggedInUserId());
		for (Integer id : items)
		{
			if (id != null)
			{
				Ownable item = (Ownable)itemType.getById(dc, id);
				if (item.hasPermission(Permission.SET_OWNER))
				{
					item.setOwner(newOwner);
					numOwned++;
				}
			}
		}
		return numOwned;
	}

}
