/**
	$Id: ExtensionPointContext.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) Authors contributing to this file.

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.util.extensions;

/**
	Invokation context for an extension point.

	@author nicklas
	@version 2.7
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
*/
class ExtensionPointContext<A extends Action>
	extends InvokationContext<A>
{
	
	private final ClientContext clientContext;
	private final Registry registry;
	private final ExtensionPoint<? super A> extensionPoint;
	private Renderer<? super A> renderer;
	private boolean hasCreatedRenderer;
	
	ExtensionPointContext(Registry registry, ClientContext clientContext, 
			ExtensionPoint<? super A> extensionPoint)
	{
		this.registry = registry;
		this.clientContext = clientContext;
		this.extensionPoint = extensionPoint;
	}
	
	/*
		From the InvokationContext class
		--------------------------------
	*/
	@Override
	public ClientContext getClientContext()
	{
		return clientContext;
	}
	
	@Override
	public ExtensionPoint<? super A> getExtensionPoint()
	{
		return extensionPoint;
	}
	
	/**
		Always null, since this is an extension point context.
	*/
	@Override
	public Extension<A> getExtension()
	{
		return null;
	}
	
	/**
		Always null, since this is an extension point context.
	*/
	@Override
	public Object getAttribute(String name)
	{
		return null;
	}

	@Override
	protected Registry getRegistry()
	{
		return registry;
	}
	
	@Override
	protected void prepareRendererFactory()
	{
		RendererFactory<? super A> factory = extensionPoint.getRendererFactory();
		if (factory != null) 
		{
			try
			{
				factory.prepareContext(this);
			}
			catch (Throwable t)
			{
				log.error("Error preparing renderer factory for extension point '" +
					extensionPoint.getId() + "': " + factory, t);
			}
		}
	}
	
	/**
		If the extension point defines a renderer factory, creates
		a new renderer unless one already exists.
		@return A renderer, or null if no factory has been defined
	*/
	@Override
	protected Renderer<? super A> getRenderer()
	{
		if (!hasCreatedRenderer)
		{
			RendererFactory<? super A> factory = extensionPoint.getRendererFactory();
			if (factory != null)
			{
				renderer = factory.getRenderer(this);
			}
			hasCreatedRenderer = true;
		}
		return renderer;
	}

	/**
		Always null, since this is an extension point context.
	*/
	@Override
	protected A[] getActions()
	{
		return null;
	}
	// ----------------------------------------

	
}
