/*
	$Id: JepExpressionFunction.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.util.jep;

import net.sf.basedb.core.query.Expression;

import org.nfunk.jep.Node;

/**
	This interface should be implemented by JEP functions
	that can also be converted into an {@link Expression} that
	can be used in queries.

	@author Nicklas
	@version 2.0
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
	@see Jep#formulaToExpression(String, JepFunction[])
*/
public interface JepExpressionFunction
	extends JepFunction
{

	/**
		Convert this function to an {@link Expression} that can be used
		in a query. The implementation must check that the node contains
		the correct number of children (ie. arguments to the function)
		and convert the arguments to whatever is suitable for the expression.
		Example from the {@link Log2Function}
		<pre class="code">
int numChildren = node.jjtGetNumChildren();
if (numChildren != 1)
{
   throw new BaseException("Invalid number of arguments for 'log2' function: " + numChildren);
}
return Expressions.log2(Jep.nodeToExpression(node.jjtGetChild(0)));
</pre>

		@param node The node representing this function
		@return An <code>Expression</code> object
	*/
	public Expression toExpression(Node node);
}
