/*
	$Id$
	
	Copyright (C) 2005 Johan Enell, Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg
	
	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/
	
	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.
	
	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/

import net.sf.basedb.core.ArrayDesign;
import net.sf.basedb.core.ArrayDesignBlock;
import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.BlockInfo;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.ItemResultList;
import net.sf.basedb.core.Platform;

 
/**
 
 @base.modified $Date$
 @author Enell
 @version 2.0
 */
public class TestArrayDesignBlock
{

	static boolean ok = true;

	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		ok = test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing array design block");
		write_header();
		int arrayDesignId = TestArrayDesign.test_create(Platform.GENERIC, false);
		// Standard tests: create, load, list
		int nbr = 10;
		int[] id = test_create(arrayDesignId, true, nbr);
		test_load(id[(int) (Math.random()*id.length)]);
		test_list(arrayDesignId, nbr);
		
		if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();

		TestArrayDesign.test_delete(arrayDesignId);
		write("++Testing array design block "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}
	
	static int[] test_create(int arraydesign_id, boolean setAll, int blocks)
	{
		int[] id = new int[blocks];
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ArrayDesign ad = ArrayDesign.getById(dc, arraydesign_id);
			ad.setName("Array design with blocks");
			
			for (int i = 0; i < blocks; i++)
			{
				int block = i+1;
				BlockInfo bi = new BlockInfo(block, (int)Math.ceil(block/4F), ((block-1)%4)+1);
				ArrayDesignBlock adb = ad.addArrayDesignBlock(bi);
				if(setAll)
				{
					adb.setBlockSizeX(10);
					adb.setBlockSizeY(10);
				}
				id[i] = adb.getId();
				write_item(0, adb);
			}
			dc.commit();
			write("--Create array design block OK");
		}
		catch (Throwable ex)
		{
			write("--Create array design block FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}

	static void test_load(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ArrayDesignBlock adb = ArrayDesignBlock.getById(dc, id);
			write_item(0, adb);
			write("--Load array design block OK");
		}
		catch (Throwable ex)
		{
			write("--Load array design block FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		
	}

	static void test_list(int arrayDesignId, int expectedResults)
	{
		if (arrayDesignId == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ArrayDesign ad = ArrayDesign.getById(dc, arrayDesignId);
			ItemResultList<ArrayDesignBlock> l = ad.getArrayDesignBlocks().list(dc);
			for (int i = 0; i<l.size(); i++)
			{
				write_item(i, l.get(i));
			}
			if (expectedResults >= 0 && expectedResults != l.size())
			{
				throw new BaseException("Expected "+expectedResults+" results, not "+l.size());
			}
			write("--List array design blocks OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--List array design blocks FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void write_header()
	{
		if (!TestUtil.getSilent())
		{
			write("  \tID\tBlock\tMetaGridX\tMetaGridY\tBlockSizeX\tBlockSizeY\tArrayDesign");
			write("--\t--\t-----\t---------\t---------\t----------\t----------\t-----------");
		}
	}
	
	static void write_item(int i, ArrayDesignBlock adb)
		throws BaseException
	{
		if (!TestUtil.getSilent()) System.out.println(i+":\t"+adb.getId()+"\t"+adb.getBlockNumber()
				+"\t"+adb.getMetaGridX()+"\t"+adb.getMetaGridY()+"\t"+adb.getBlockSizeX()+"\t"+adb.getBlockSizeY()+"\t"+adb.getArrayDesign().getId());
	}
	
	static void write(String message)
	{
		System.out.println(message);
	}
}
