import java.util.Date;

import net.sf.basedb.core.ArrayBatch;
import net.sf.basedb.core.ArraySlide;
import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.Permission;
import net.sf.basedb.core.ItemResultList;
import net.sf.basedb.core.Platform;
import net.sf.basedb.core.Trashcan;

/*
	$Id$
	
	Copyright (C) 2005 Johan Enell, Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen
	
	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/
	
	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.
	
	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
public class TestArraySlide
{
	static boolean ok = true;
	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		ok = test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing array slide");
		write_header();
		// Standard tests: create, load, list
		int arraydesign_id = TestArrayDesign.test_create(Platform.GENERIC, true);
		int arraybatch_id = TestArrayBatch.test_create(arraydesign_id, false);
		int id = test_create(arraybatch_id, true);
		int id2 = test_create(arraybatch_id, false);
		test_load(id);
		test_list(-1);

		// Extra tests:

		if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();
		// Standard test: Delete
		test_delete(id);
		test_delete(id2);
		TestArrayBatch.test_delete(arraybatch_id);
		TestArrayDesign.test_delete(arraydesign_id);
		write("++Testing array slide "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}

	static int test_create(int arraybatch_id, boolean setAll)
	{
		if (arraybatch_id == 0 || !TestUtil.hasPermission(Permission.CREATE, Item.ARRAYSLIDE)) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ArrayBatch ab = ArrayBatch.getById(dc, arraybatch_id);
			ArraySlide as = ab.newArraySlide();
			if (setAll)
			{
				as.setName("Test array slide");
				as.setDescription("Added at "+new Date());
				as.setBatchIndex(1);
				as.setBarcode("ABCDEFG");
			}
			dc.saveItem(as);
			dc.commit();
			id = as.getId();
			dc = TestUtil.getDbControl();
			dc.reattachItem(as);
			write_item(0, as);
			write("--Create array slide OK");
		}
		catch (Exception ex)
		{
			write("--Create array slide FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}

	static void test_load(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ArraySlide as = ArraySlide.getById(dc, id);
			write_item(0, as);
			write("--Load array slide OK");
		}
		catch (Exception ex)
		{
			write("--Load array slide FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_list(int expectedResults)
	{
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ItemResultList<ArraySlide> l = ArraySlide.getQuery().list(dc);
			for (int i = 0; i<l.size(); i++)
			{
				write_item(i, l.get(i));
			}
			if (expectedResults >= 0 && expectedResults != l.size())
			{
				throw new BaseException("Expected "+expectedResults+" results, not "+l.size());
			}
			write("--List array slides OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--List array slides FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}


	static void test_delete(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ArraySlide as = ArraySlide.getById(dc, id);
			dc.deleteItem(as);
			dc.commit();
			write("--Delete array slide OK");
		}
		catch (Exception ex)
		{
			write("--Delete array slide FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_delete_all()
	{
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ItemResultList<ArraySlide> l = ArraySlide.getQuery().list(dc);
			Trashcan.delete(dc.getSessionControl(), l, true, null);
			dc.commit();
			write("--Delete all array slides OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--Delete all array slides FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	
	static void write_header()
	{
		if (!TestUtil.getSilent())
		{
			write("   \tID \tName      \tDescription\tIndex\tBarcode\tArray batch");
			write("-- \t-- \t--------- \t-----------\t-----\t-------\t-----------");
		}
	}
	static void write_item(int i, ArraySlide as)
		throws BaseException
	{
		if (!TestUtil.getSilent()) System.out.println(i+":\t"+as.getId()+"\t"+as.getName()+"\t"+
			as.getDescription()+"\t"+as.getBatchIndex()+"\t"+as.getBarcode()+"\t"+
			as.getArrayBatch());
	}
	static void write(String message)
	{
		System.out.println(message);
	}
}
