import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.BioAssaySet;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Directory;
import net.sf.basedb.core.Experiment;
import net.sf.basedb.core.File;
import net.sf.basedb.core.Formula;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.Job;
import net.sf.basedb.core.ParameterType;
import net.sf.basedb.core.Permission;
import net.sf.basedb.core.PluginConfiguration;
import net.sf.basedb.core.PluginConfigurationRequest;
import net.sf.basedb.core.PluginDefinition;
import net.sf.basedb.core.PluginParameter;
import net.sf.basedb.core.PluginResponse;
import net.sf.basedb.core.RequestInformation;
import net.sf.basedb.core.Type;
import net.sf.basedb.core.plugin.Response;

/*
 $Id$

 Copyright (C) 2006 Johan Enell
 Copyright (C) 2007 Nicklas Nordborg

 This file is part of BASE - BioArray Software Environment.
 Available at http://base.thep.lu.se/

 BASE is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 3 of the License, or
 (at your option) any later version.

 BASE is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 02111-1307, USA.
*/

public class TestBase1PluginExecuter
{
	static boolean ok = true;
	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		ok = test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing TestBase1PluginExecuter");
		
		// Import reporters
		TestReporter.test_import_from_file("data/test.rawdata.import.txt", 
			"\"Block\"\\t\"Column\"\\t\"Row\"\\t\"Name\"\\t\"ID\".*", "\\t", 4, 3);
		
		// Create experiment
		int experimentId = TestAnalyzePluginUtil.test_createExperiment("data/test.rawdata.import.txt", 2, 100, 0.8f);
		int[] rawBioAssays = TestExperiment.test_list_rawbioassays(experimentId, -1);

		int formulaId = TestFormula.test_create("raw fg mean", Formula.Type.INTENSITY_EXPRESSION,
				"genepix", new String[] { "raw('ch1FgMedian')", "raw('ch2FgMedian')" }, false);
		
		int formulaChromosomeId = TestFormula.test_create("chromosome", 
				Formula.Type.COLUMN_EXPRESSION, "genepix", new String[] { "rep('chromosome')" }, false);
		int posFormulaId = TestFormula.test_create("position",
				Formula.Type.COLUMN_EXPRESSION, "genepix", new String[] { "pos()" }, false);
		int reporterFormulaId = TestFormula.test_create("reporter",
				Formula.Type.COLUMN_EXPRESSION, "genepix", new String[] { "rep('id')" }, false);
		int externalIdFormulaId = TestFormula.test_create("reporterId",
				Formula.Type.COLUMN_EXPRESSION, "genepix", new String[] { "rep('externalId')" }, false);
		
		int fileId = TestFile.test_create("data/test.nullplugin.base", false, false);
		int fileId2 = TestFile.test_create("data/test.base1plugin.base", false, false);
		int fileIdMatrix = TestFile.test_create("data/test.nullplugin.matrix.base", false, false);
		int directoryId = TestDirectory.test_create(true, "Base1PluginExecuter");
		
		int rootBioAssaySetId = TestAnalyzePluginUtil.test_createRootBioAssaySet(experimentId, formulaId);
		int filteredBioAssaySetId = TestExperiment.test_filter_bioassayset(rootBioAssaySetId, 1);
		int pluginDefinitionId = TestPluginDefinition.test_get("net.sf.basedb.plugins.Base1PluginExecuter");
		int pluginConfigurationId = test_create_configuration(pluginDefinitionId, fileId);
		int pluginConfigurationId2 = test_create_configuration(pluginDefinitionId, fileId2);
		int pluginConfigurationIdMatrix = test_create_configuration(pluginDefinitionId, fileIdMatrix);
		
		// Calculate extra value
		int aExtraValueId = TestExtraValueType.test_create("a", Type.FLOAT, false);
		TestExperiment.test_calculate_spotextravalues(rootBioAssaySetId, false, 0, aExtraValueId, 0);
		TestExperiment.test_calculate_spotextravalues(filteredBioAssaySetId, false, 0, aExtraValueId, 0);
		
		// Copy input to output - serial format
		int jobId = test_create_job(pluginConfigurationId, experimentId, rootBioAssaySetId, directoryId);
		ok &= TestJob.test_execute(jobId);
		
		// Copy input to output - matrix format
		int jobIdMatrix = test_create_job(pluginConfigurationIdMatrix, experimentId, rootBioAssaySetId, directoryId);
		ok &= TestJob.test_execute(jobIdMatrix);
		
		// Copy input to output - matrix format - filtered bioassayset
		int jobIdMatrix2 = test_create_job(pluginConfigurationIdMatrix, experimentId, filteredBioAssaySetId, directoryId);
		ok &= TestJob.test_execute(jobIdMatrix2);
		
		// One-to-one mapping on assays and reporters
		int jobId2 = test_create_job(pluginConfigurationIdMatrix, experimentId, rootBioAssaySetId, directoryId);
		ok &= TestJob.test_execute(jobId2);

		// Merge assays
		int jobId3 = test_create_job(pluginConfigurationId2, experimentId, rootBioAssaySetId, directoryId, 
				new Parameter("mergeAssays", "1"));
		ok &= TestJob.test_execute(jobId3);

		// Offset positions
		int jobId4 = test_create_job(pluginConfigurationId2, experimentId, rootBioAssaySetId, directoryId, 
				new Parameter("offsetPositions", "1"));
		ok &= TestJob.test_execute(jobId4);

		// Offset positions and merge assays
		int jobId5 = test_create_job(pluginConfigurationId2, experimentId, rootBioAssaySetId, directoryId, 
				new Parameter("mergeAssays", "1"), new Parameter("offsetPositions", "1"));
		ok &= TestJob.test_execute(jobId5);

		// Create reporter list
		int jobId6 = test_create_job(pluginConfigurationId2, experimentId, rootBioAssaySetId, directoryId,
				new Parameter("reporterList", "1"));
		ok &= TestJob.test_execute(jobId6);
		
		if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();
		
		// Cleanup
		TestExperiment.test_delete(experimentId);
		if (rawBioAssays != null)
		{
			for (int rbaId : rawBioAssays)
			{
				TestRawBioAssay.test_delete(rbaId);
			}
		}
		TestExtraValueType.test_delete(aExtraValueId);
		TestPluginConfiguration.test_delete(pluginConfigurationId);
		TestPluginConfiguration.test_delete(pluginConfigurationId2);
		TestPluginConfiguration.test_delete(pluginConfigurationIdMatrix);
		TestFormula.test_delete(formulaId);
		TestFormula.test_delete(formulaChromosomeId);
		TestFormula.test_delete(posFormulaId);
		TestFormula.test_delete(reporterFormulaId);
		TestFormula.test_delete(externalIdFormulaId);
		TestFile.test_delete(fileId);
		TestFile.test_delete(fileId2);
		TestFile.test_delete(fileIdMatrix);
		TestDirectory.test_delete(directoryId, true);
		TestReporter.test_delete();
		write("++Testing TestBase1PluginExecuter "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}
	
	static int test_create_configuration(int pluginDefinitionId, int fileId)
	{
		if (pluginDefinitionId == 0 || !TestUtil.hasPermission(Permission.CREATE, Item.PLUGINCONFIGURATION)) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();

			PluginDefinition pd = PluginDefinition.getById(dc, pluginDefinitionId);
			PluginConfiguration pc = pd.newPluginConfiguration();
			pc.setName(pd.getName());

			PluginConfigurationRequest request = pc.configure();
			request.setParameterValue("execPath", new java.io.File("").getAbsolutePath()+java.io.File.separator);
			request.setParameterValue("file", File.getById(dc, fileId));

			write_request_information(request.getRequestInformation());
			
			PluginResponse response = request.invoke();
			if (response.getStatus() == Response.Status.DONE)
			{
				response.saveParameters(dc);
				dc.saveItem(pc);
				dc.commit();
			}
			else
			{
				throw new BaseException(response.getMessage(), response.getErrorList().get(0));
			}
			id = pc.getId();
			write("--Create configuration for Base1PluginExecuter OK");
		}
		catch (Throwable ex)
		{
			write("--Create configuration for Base1PluginExecuter FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}
	
	static int test_create_job(int pluginConfigurationId, int experimentId, int bioAssaySetId, int directoryId, Parameter... parameters)
	{
		if (pluginConfigurationId == 0 || !TestUtil.hasPermission(Permission.CREATE, Item.JOB)) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();

			Experiment e = Experiment.getById(dc, experimentId);
			BioAssaySet bioAssaySet = BioAssaySet.getById(dc, bioAssaySetId);
			
			PluginConfiguration pc = PluginConfiguration.getById(dc, pluginConfigurationId);
			Job j = pc.newJob(e);
			j.setName(pc.getName());
			
			PluginConfigurationRequest request = j.configure(null);
			
			// First, set default values
			for (PluginParameter<?> p : request.getRequestInformation().getParameters())
			{
				request.setParameterValue(p.getName(), p.getParameterType().getDefaultValue());
			}
			
			// Then, set values from method call
			if (parameters != null)
			{
				for (Parameter p : parameters)
				{
					request.setParameterValue(p.name, p.value);
				}
			}
			
			request.setParameterValue("source", bioAssaySet);
			request.setParameterValue("pluginDirectory", Directory.getById(dc, directoryId).getPath().toString());

			write_request_information(request.getRequestInformation());
			
			PluginResponse response = request.invoke();
			if (response.getStatus() == Response.Status.DONE)
			{
				response.saveParameters(dc);
				dc.saveItem(j);
				dc.commit();
			}
			else
			{
				throw new BaseException(response.getMessage(), response.getErrorList().get(0));
			}
			id = j.getId();
			write("--Create job for Base1PluginExecuter OK");
		}
		catch (Throwable ex)
		{
			write("--Create job for Base1PluginExecuter FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}

	static void write(String message)
	{
		System.out.println(message);
	}
	
	static void write_request_information(RequestInformation ri)
	{
		if (TestUtil.getSilent()) return;

		write("Request information for command: "+ri.getCommand());
		write("\tTitle:\t"+ri.getTitle());
		write("\tDescription:\t"+ri.getDescription());
		for (PluginParameter<?> pp : ri.getParameters())
		{
			ParameterType pt = pp.getParameterType();
			System.out.println("\tParameter:\t"+pp.getName() + "\t" + pp.getLabel() + "\t" + pt );
		}
	}
	
	static class Parameter
	{
		final String name;
		final Object value;
		Parameter(String name, Object value)
		{
			this.name = name;
			this.value = value;
		}
	}
	
}
