import net.sf.basedb.core.Annotation;
import net.sf.basedb.core.AnnotationSet;
import net.sf.basedb.core.AnnotationType;
import net.sf.basedb.core.BioAssay;
import net.sf.basedb.core.BioAssaySet;
import net.sf.basedb.core.Config;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Directory;
import net.sf.basedb.core.Experiment;
import net.sf.basedb.core.File;
import net.sf.basedb.core.Formula;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.ItemQuery;
import net.sf.basedb.core.Platform;
import net.sf.basedb.core.RawBioAssay;
import net.sf.basedb.core.RawDataBatcher;
import net.sf.basedb.core.RawDataTypes;
import net.sf.basedb.core.Reporter;
import net.sf.basedb.core.ReporterBatcher;
import net.sf.basedb.core.SystemItems;
import net.sf.basedb.core.Type;
import net.sf.basedb.core.data.RawData;
import net.sf.basedb.core.data.ReporterData;
import net.sf.basedb.core.query.Expressions;
import net.sf.basedb.core.query.Hql;
import net.sf.basedb.core.query.Restrictions;
import net.sf.basedb.plugins.BioAssaySetExporter;
import net.sf.basedb.util.parser.FlatFileParser;
import net.sf.basedb.util.parser.FlatFileParser.Line;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashSet;
import java.util.Set;
import java.util.TreeMap;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/*
 $Id$

 Copyright (C) 2007 Johan Enell

 This file is part of BASE - BioArray Software Environment.
 Available at http://base.thep.lu.se/

 BASE is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 3 of the License, or
 (at your option) any later version.

 BASE is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 02111-1307, USA.
 */

/**

 @author johan
 @version 2.3
 @base.modified $Date$
 */
public class TestBioAsssaySetExporter
{
	static boolean ok = true;
	
	static ArrayList<String> readlines = new ArrayList<String>();
	
	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		ok = test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing BioAsssaySetExporter");
		
		int r1 = test_createReporter("exportreporter1");
		int r2 = test_createReporter("exportreporter2");
		
		int rba1 = 0;
		int rba2 = 0;
		int annotationType = test_createAnnotationType("testExportAnnotation");
		if (ok)
		{
			rba1 = test_createRawBioAssay("export1", annotationType, "1");
			rba2 = test_createRawBioAssay("export2", annotationType, "2");
			test_addRawData(rba1, r1, r2, r1);
			test_addRawData(rba2, r1, r2, r2);
		}

		int experiment = 0;
		int file = 0;
		int bas = 0;
		if (ok)
		{
			file = test_createFile("testexport.base");
			experiment = TestExperiment.test_create("genepix", false);
			test_addExperimentalFactor(experiment, annotationType);
			TestExperiment.test_add_rawbioassay(experiment, rba1);
			TestExperiment.test_add_rawbioassay(experiment, rba2);
			bas = TestExperiment.test_create_root_bioassayset_using_calculator(experiment, "medianfg", new int[] {rba1, rba2});
			
			test_exportBaseFile(bas, file, false, false);
			test_exportBaseFile(bas, file, true, false);
			test_exportBaseFile(bas, file, false, true);
			test_exportBaseFile(bas, file, true, true);
		}
		
		if (experiment > 0)
			TestExperiment.test_delete(experiment);
		if (rba1 > 0)
			TestRawBioAssay.test_delete(rba1);
		if (rba1 > 0)
			TestRawBioAssay.test_delete(rba2);
		if (file > 0)
			TestFile.test_delete(file);
		if (annotationType > 0)
			TestAnnotationType.test_delete(annotationType);
		test_deleteReporter(r1, r2);
		
		return ok;
	}
	
	private static int test_addExperimentalFactor(int experimentId, int annotationTypeId)
	{
		DbControl dc = null;
		int id = 0;
		try
		{
			dc = TestUtil.getDbControl();
			Experiment e = Experiment.getById(dc, experimentId);
			AnnotationType at = AnnotationType.getById(dc, annotationTypeId);
			e.addExperimentalFactor(at);
			dc.commit();
			write("--Experimental factor was added to experiment.");
		}
		catch (Throwable ex)
		{
			write("--Add experimental factor FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}

	private static void test_addRawData(int rbaId, int r1Id, int r2Id, int r3Id)
	{
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			
			RawBioAssay rba = RawBioAssay.getById(dc, rbaId);
			ReporterData r1 = Reporter.getById(dc, r1Id);
			ReporterData r2 = Reporter.getById(dc, r2Id);
			ReporterData r3 = Reporter.getById(dc, r3Id);
			RawDataBatcher rdb = rba.getRawDataBatcher(null, null);
			for (int i=0; i < 4; ++i)
			{
				RawData rd = rdb.newRawData();
				rd.setBlock(1);
				rd.setMetaGridX(0);
				rd.setMetaGridY(0);
				rd.setColumn(i%2);
				rd.setRow((int)Math.floor(i/2));
				if (i < 2) rd.setReporter(r1);
				else rd.setReporter(r2);
				rd.setExtended("flags", Integer.valueOf(0));
				rd.setExtended("ch1FgMedian", Float.valueOf(1));
				rd.setExtended("ch2FgMedian", Float.valueOf(2));
				rdb.insert(rd);
			}
			
			// the last position
			RawData rd = rdb.newRawData();
			rd.setBlock(1);
			rd.setMetaGridX(0);
			rd.setMetaGridY(0);
			rd.setColumn(0);
			rd.setRow(2);
			rd.setReporter(r3);
			rd.setExtended("flags", Integer.valueOf(-50));
			rd.setExtended("ch1FgMedian", Float.valueOf(10));
			rd.setExtended("ch2FgMedian", Float.valueOf(20));
			rdb.insert(rd);
			
			rdb.close();
			dc.commit();
			write("--RawData was added to RawBioAssay "+rba.toString());
		}
		catch (Throwable ex)
		{
			write("--Add rawdata FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	private static int test_createAnnotationType(String name)
	{
		DbControl dc = null;
		int id = 0;
		try
		{
			dc = TestUtil.getDbControl();
			AnnotationType at = AnnotationType.getNew(dc, Type.STRING);
			at.setName(name);
			at.enableForItem(Item.RAWBIOASSAY);
			dc.saveItem(at);
			dc.commit();
			id = at.getId();
			write("--AnnotationType created");
		}
		catch (Throwable ex)
		{
			write("--Create AnnotationType FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}
	
	private static int test_createFile(String name)
	{
		DbControl dc = null;
		int id = 0;
		try
		{
			dc = TestUtil.getDbControl();
			File file = File.getNew(dc, Directory.getById(dc, SystemItems.getId(Directory.ROOT)));
			file.setName(name);
			dc.saveItem(file);
			dc.commit();
			id = file.getId();
		}
		catch (Throwable ex)
		{
			write("--Create file FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}
	
	private static int test_createRawBioAssay(String name, int annotationTypeId, String annotationValue)
	{
		DbControl dc = null;
		int id = 0;
		try
		{
			dc = TestUtil.getDbControl();
			Platform generic = Platform.getByExternalId(dc, Platform.GENERIC);
			RawBioAssay rba = RawBioAssay.getNew(dc, generic, RawDataTypes.getSafeRawDataType("genepix"));
			AnnotationType at = AnnotationType.getById(dc, annotationTypeId);
			
			rba.setName(name);
			dc.saveItem(rba);
			
			AnnotationSet as = rba.getAnnotationSet();
			Annotation a = as.getAnnotation(at);
			
			a.setValue(annotationValue);
			
			dc.commit();
			id = rba.getId();
			dc = TestUtil.getDbControl();
			dc.reattachItem(rba);
			write("--Create raw bioassay OK");
		}
		catch (Throwable ex)
		{
			write("--Create raw bioassay FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}
	
	private static int test_createReporter(String name)
	{
		DbControl dc = null;
		int id = 0;
		try
		{
			dc = TestUtil.getDbControl();
			ReporterBatcher rb = ReporterBatcher.getNew(dc);
			ReporterData r = Reporter.getNew(name);
			rb.insert(r);
			rb.close();
			dc.commit();
			
			dc = TestUtil.getDbControl();
			r = Reporter.getByExternalId(dc, name);
			id = r.getId();
			write("--Created reporter "+name+" with id("+id+")");
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}
	
	private static void test_deleteReporter(int... ids)
	{
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ReporterBatcher rb = ReporterBatcher.getNew(dc);
			for (int r : ids)
			{
				if (r > -1) rb.delete(r);
			}
			rb.close();
			dc.commit();
			write("--Deleted reporters "+Arrays.toString(ids));
		}
		catch (Throwable ex)
		{
			write("--Deleted reporters "+Arrays.toString(ids) +" FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	/**
		The BioAssaySet thats beeing tested must be the one created by this class.
	 */
	private static void test_exportBaseFile(int bioAssaySetId, int fileId, boolean matrix, boolean mergeReporters)
	{
		DbControl dc = null;
		FlatFileParser ffp = null;
		try
		{
			dc = TestUtil.getDbControl();
			BioAssaySet bas = BioAssaySet.getById(dc, bioAssaySetId);
			File file = File.getById(dc, fileId);
			
			// I use a TreeMap so I can predict the order of the parameters
			TreeMap<String, String> parameters = new TreeMap<String, String>();
			parameters.put("b", "some parameter");
			parameters.put("a", "some other parameter");
			
			ArrayList<String> rep = new ArrayList<String>();
			rep.add("rep.id");
			rep.add("rep.name");
			rep.add("rep.position");
			
			ArrayList<String> spot = new ArrayList<String>();
			ItemQuery<Formula> ratio = Formula.getQuery(Formula.Type.COLUMN_EXPRESSION, bas.getRawDataType());
			ratio.restrict(Restrictions.eq(Hql.property("name"), Expressions.string("Ratio, ch1 / ch2")));
			if (ratio.count(dc) < 1)
			{
				throw new Exception("Cant find formula 'Ratio, ch1 / ch2'");
			}
			spot.add("intensity1");
			spot.add("raw.row");
			spot.add("raw.col");
			spot.add("raw.flags");
			spot.add("frm."+ratio.list(dc).get(0).getId());
			
			//Create the expected header values values
			//Assays section
			Set<String> assaysAnnotationColumns = Collections.singleton("testExportAnnotation");
			HashSet<String> assaysColumns = new HashSet<String>(assaysAnnotationColumns);
			assaysColumns.add("id");
			assaysColumns.add("name");
			Set<String> assaysCount = Collections.singleton("2");
			
			//Spots section
			Set<String> spotsChannels = Collections.singleton("2");
			HashSet<String> spotsAssayFields = new HashSet<String>();
			spotsAssayFields.add("intensity1");
			spotsAssayFields.add("row");
			spotsAssayFields.add("col");
			spotsAssayFields.add("flags");
			spotsAssayFields.add("Ratio, ch1 / ch2");
			HashSet<String> spotsColumns = new HashSet<String>();
			spotsColumns.add("reporter");
			spotsColumns.add("name");
			spotsColumns.add("position");
			spotsColumns.add("assayData");
			HashSet<String> spotsAssays = new HashSet<String>();
			for (BioAssay ba : bas.getBioAssays().list(dc)) spotsAssays.add(String.valueOf(ba.getId()));
			Set<String> spotsCount;
			if (matrix)
			{
				if (mergeReporters) spotsCount = Collections.singleton("2");
				else spotsCount = Collections.singleton("6");
			}
			else
			{
				if (mergeReporters) spotsCount = Collections.singleton("2");
				else spotsCount = Collections.singleton("5");
			}
			
			BioAssaySetExporter exporter = new BioAssaySetExporter();
			if (matrix)
			{
				exporter.exportBaseFileMatrix(bas, file, parameters, rep, spot, mergeReporters);
			}
			else
			{
				exporter.exportBaseFileSerial(bas, file, parameters, rep, spot, mergeReporters);
			}
			
			ffp = new FlatFileParser();
			ffp.setSectionRegexp(Pattern.compile("section[\\t](.*)"));
			ffp.setHeaderRegexp(Pattern.compile("(.*?)\\t(.*)"));
			ffp.setDataHeaderRegexp(Pattern.compile("%"));
			ffp.setDataFooterRegexp(Pattern.compile("^$"));
			ffp.setDataSplitterRegexp(Pattern.compile("\\t"));
			ffp.setInputStream(file.getDownloadStream(0), Config.getCharset());
			ffp.setUseNullIfEmpty(false);

			ArrayList<String[]> rows = new ArrayList<String[]>();
			
			readlines.clear();
			// Test settings section
			ffp.parseHeaders();
			for (Line l : ffp.getLines()) readlines.add(l.toString());
			TestFlatFileParser.testLine(ffp.getLine(0), "BASEfile");
			TestFlatFileParser.testSection(ffp.nextSection(), "settings");
			ffp.parseHeaders();
			for (Line l : ffp.getLines()) readlines.add(l.toString());
			TestFlatFileParser.testHeaderLine(ffp.getLine(0), "a", "some other parameter");
			TestFlatFileParser.testHeaderLine(ffp.getLine(1), "b", "some parameter");
			
			
			readlines.clear();
			// Test assays section
			TestFlatFileParser.testSection(ffp.nextSection(), "assays");
			ffp.parseHeaders();
			for (Line l : ffp.getLines()) readlines.add(l.toString());
			
			test_baseFileSectionHeader("annotationColumns", ffp.getHeader("annotationColumns"), assaysAnnotationColumns);
			test_baseFileSectionHeader("columns", ffp.getHeader("columns"), assaysColumns);
			test_baseFileSectionHeader("count", ffp.getHeader("count"), assaysCount);
			rows.add(null);
			rows.add(null);
			test_baseFileSectionData(ffp, rows);
			
			readlines.clear();
			// Test spot section(s)
			int numOfSections = matrix ? 1 : (int) bas.getBioAssays().count(dc);
			Line section = ffp.nextSection();
			while (section != null && numOfSections > 0)
			{
				TestFlatFileParser.testSection(section, "spots");
				ffp.parseHeaders();
				for (Line l : ffp.getLines()) readlines.add(l.toString());
				test_baseFileSectionHeader("channels", ffp.getHeader("channels"), spotsChannels);
				test_baseFileSectionHeader("assayFields", ffp.getHeader("assayFields"), spotsAssayFields);
				test_baseFileSectionHeader("columns", ffp.getHeader("columns"), spotsColumns);
				test_baseFileSectionHeader("assays", ffp.getHeader("assays"), spotsAssays);
				test_baseFileSectionHeader("count", ffp.getHeader("count"), spotsCount);

				rows.clear();
				if (matrix)
				{
					if (mergeReporters)
					{
						String[] cols = {"\\d+","New reporter","","[14]","","","","0\\.5","[14]","","","","0\\.5"};
						rows.add(cols);
						rows.add(cols);
					}
					else
					{
						String[] cols1 = {"\\d+","New reporter","[1234]","1","[01]","[01]","0","0\\.5","1","[01]","[01]","0","0\\.5"};
						String[] cols2 = {"\\d+","New reporter","[56]","(10)|","2|","0|","-50|","0\\.5|","(10)|","2|","0|","-50|","0\\.5|"};
						rows.add(cols1);
						rows.add(cols1);
						rows.add(cols1);
						rows.add(cols1);
						rows.add(cols2);
						rows.add(cols2);
					}
				}
				else
				{
					if (mergeReporters) 
					{
						String[] cols = {"\\d+","New reporter","","[14]","","","","0\\.5"};
						rows.add(cols);
						rows.add(cols);
					}
					else 
					{
						String[] cols = {"\\d+","New reporter","[123456]","10?","[012]","[01]","(0)|(-50)","0\\.5"};
						rows.add(cols);
						rows.add(cols);
						rows.add(cols);
						rows.add(cols);
						rows.add(cols);
					}
				}
				test_baseFileSectionData(ffp, rows);
				
				section = ffp.nextSection();
				--numOfSections;
			}
			
			write("--Exporting basefile [matrix:"+matrix+", mergeReporters:"+mergeReporters+"] SUCCEEDED");
		}
		catch (Throwable ex)
		{
			write("--Exporting basefile [matrix:"+matrix+", mergeReporters:"+mergeReporters+"] FAILED");
			for (String line : readlines)
			{
				write("["+line+"]");
			}
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	private static ArrayList<FlatFileParser.Data> test_baseFileSectionData(FlatFileParser ffp, ArrayList<String[]> rows)
		throws Exception
	{
		ArrayList<FlatFileParser.Data> datalines = new ArrayList<FlatFileParser.Data>(rows.size());
		while (ffp.hasMoreData())
		{
			FlatFileParser.Data dataline = ffp.nextData();
			readlines.add(dataline.toString());
			datalines.add(dataline);
		}
		
		if (datalines.size() != rows.size())
			throw new Exception("Wrong number of data lines. Found "+datalines.size()+" expected "+rows.size());
		
		for (int i = 0; i < rows.size(); ++i)
		{
			FlatFileParser.Data dataline = datalines.get(i);
			
			String[] row = rows.get(i);
			if (row != null)
			{
				if (dataline.columns() != row.length)
					throw new Exception("Wrong number of columns. Found "+dataline.columns()+" expected "+row.length+". ["+dataline+"]");
				
				for (int j = 0; j < row.length; ++j)
				{
					String col = row[j];
					Pattern p = Pattern.compile(col);
					Matcher m = p.matcher(dataline.get(j));
					if (!m.matches())
					{
						throw new Exception("The value["+dataline.get(j)+"] at column number "+(j+1)+" dosent match the pattern '"+p+"'. ["+dataline+"]");
					}
				}
			}
		}
		
		return datalines;
	}

	private static void test_baseFileSectionHeader(String header, String found, Set<String> expected)
		throws Exception
	{
		HashSet<String> foundSet = new HashSet<String>(Arrays.asList(found.split("\\t")));
		if (!expected.containsAll(foundSet))
			throw new Exception("Expected values '"+expected+"' didnt contain the header values '"+foundSet+"' for header '"+header+"'");
		
	}

	static void write(String message)
	{
		System.out.println(message);
	}
	
}
