import java.util.Date;

import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.Permission;
import net.sf.basedb.core.ProtocolType;
import net.sf.basedb.core.ItemResultList;

/*
	$Id$

	Copyright (C) 2005 Johan Enell, Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/

public class TestProtocolType
{
	static boolean ok = true;
	
	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		ok = test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing protocol type");
		write_header();
		// Standard tests: create, load, list
		int id1 = test_create(true);
		int id2 = test_create(false);
		test_load(id1);
		test_list();

		// Standard test: Delete
		if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();
		test_delete(id1);
		test_delete(id2);
		write("++Testing protocol type "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}

	static int test_create(boolean setAll)
	{
		if (!TestUtil.hasPermission(Permission.CREATE, Item.PROTOCOLTYPE)) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ProtocolType pt = ProtocolType.getNew(dc);
			if (setAll)
			{
				pt.setName("Test protocol type");
				pt.setDescription("Added at "+new Date());
			}
			dc.saveItem(pt);
			dc.commit();
			id = pt.getId();
			write_item(0, pt);
			write("--Create protocol type OK");
		}
		catch (Throwable ex)
		{
			write("--Create protocol type FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}

	static void test_load(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ProtocolType pt = ProtocolType.getById(dc, id);
			write_item(0, pt);
			write("--Load protocol type OK");
		}
		catch (Throwable ex)
		{
			write("--Load protocol type FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_list()
	{
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ItemResultList<ProtocolType> l = ProtocolType.getQuery().list(dc);
			for (int i = 0; i<l.size(); i++)
			{
				write_item(i, l.get(i));
			}
			write("--List protocol types OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--List protocol types FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	static void test_delete(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ProtocolType pt = ProtocolType.getById(dc, id);
			dc.deleteItem(pt);
			dc.commit();
			write("--Delete protocol type OK");
		}
		catch (Throwable ex)
		{
			write("--Delete protocol type FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	static void write_header()
	{
		if (!TestUtil.getSilent())
		{
			write("   \tID \tName      \tDescription\tSystem");
			write("-- \t-- \t--------- \t-----------\t------");
		}
	}
	
	static void write_item(int i, ProtocolType pt)
		throws BaseException
	{
		if (!TestUtil.getSilent()) 
			System.out.println(i+":\t"+pt.getId()+"\t"+pt.getName()+"\t"+pt.getDescription()+"\t"+pt.isSystemItem());
	}
	
	static void write(String message)
	{
		System.out.println(message);
	}
}
