/*
	$Id: TestQuotaType.java 4480 2008-09-05 15:16:58Z jari $

	Copyright (C) 2005 Johan Enell, Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.QuotaType;
import net.sf.basedb.core.ItemResultList;

public class TestQuotaType
{

	static boolean ok = true;
	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		ok = test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing quota types");
		write_header();

		// Standard tests: list
		test_list();

		write("++Testing quota types "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}
	
	static void test_list()
	{
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ItemResultList<QuotaType> l = QuotaType.getQuery().list(dc);
			for (int i = 0; i<l.size(); i++)
			{
				write_item(i, (QuotaType)l.get(i));
			}
			write("--List quota types OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--List quota types FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	static void test_delete(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			QuotaType q = QuotaType.getById(dc, id);
			dc.deleteItem(q);
			dc.commit();
			write("--Delete quota type OK");
		}
		catch (Throwable ex)
		{
			write("--Delete quota type FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void write_header()
	{
		if (!TestUtil.getSilent())
		{
			write("   \tID \tName\tDescription");
			write("-- \t-- \t----\t-----------");
		}
	}
	static void write_item(int i, QuotaType qt)
		throws BaseException
	{
		if (!TestUtil.getSilent())
			System.out.println(i+":\t"+qt.getId()+"\t"+
				qt.getName()+"\t"+qt.getDescription()
				);
	}
	static void write(String message)
	{
		System.out.println(message);
	}
}

