/*
	$Id: TestUserClientSetting.java 4480 2008-09-05 15:16:58Z jari $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
import net.sf.basedb.core.*;

public class TestUserClientSetting
{

	static boolean ok = true;

	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing user client setting");
		write_header();
		// Standard tests: create, load, list
		int userId = TestUser.test_create(false);
		int clientId = TestClient.test_create("net.sf.basedb.clients.testcreate", false);
		int id = test_create(userId, clientId, "fontsize", "9");
		test_duplicate(id);
		test_load(id);
		test_list(userId, clientId, -1);

		// Extra tests:

		// Standard test: Delete
		if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();
		test_delete(id);
		TestClient.test_delete(clientId);
		TestUser.test_delete(userId);
		write("++Testing user client setting "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}

	static int test_create(int userId, int clientId, String name, String value)
	{
		if (userId == 0 || clientId == 0) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			User user = User.getById(dc, userId);
			Client client = Client.getById(dc, clientId);
			UserClientSetting ucs = UserClientSetting.getNew(dc, user, client, name, value);
			dc.saveItem(ucs);
			dc.commit();
			id = ucs.getId();
			dc = TestUtil.getDbControl();
			dc.reattachItem(ucs);
			write_item(0, ucs);
			write("--Create user client setting OK");
		}
		catch (Throwable ex)
		{
			write("--Create user client setting FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}

	static void test_load(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			UserClientSetting ucs = UserClientSetting.getById(dc, id);
			write_item(0, ucs);
			write("--Load user client setting OK");
		}
		catch (Throwable ex)
		{
			write("--Load user client setting FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_list(int userId, int clientId, int expectedResults)
	{
		if (userId == 0 || clientId == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			User user = User.getById(dc, userId);
			Client client = Client.getById(dc, clientId);
			ItemResultList<UserClientSetting> l = UserClientSetting.getQuery(user, client).list(dc);
			for (int i = 0; i<l.size(); i++)
			{
				write_item(i, l.get(i));
			}
			if (expectedResults >= 0 && expectedResults != l.size())
			{
				throw new BaseException("Expected "+expectedResults+" results, not "+l.size());
			}
			write("--List user client settings OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--List user client settings FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_delete(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			UserClientSetting ucs = UserClientSetting.getById(dc, id);
			dc.deleteItem(ucs);
			dc.commit();
			write("--Delete user client setting OK");
		}
		catch (Throwable ex)
		{
			write("--Delete user client setting FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void write_header()
	{
		if (!TestUtil.getSilent())
		{
			write("   \tID \tName\tValue\tUser\tClient");
			write("-- \t-- \t----\t-----\t----\t------");
		}
	}
	static void write_item(int i, UserClientSetting ucs)
		throws BaseException
	{
		if (!TestUtil.getSilent()) System.out.println(i+":\t"+ucs.getId()+"\t"+ucs.getName()+"\t"+
			ucs.getValue()+"\t"+ucs.getUser()+"\t"+ucs.getClient());
	}
	static void write(String message)
	{
		System.out.println(message);
	}
	
	static void test_duplicate(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			UserClientSetting ucs = UserClientSetting.getById(dc, id);
			UserClientSetting duplicate = UserClientSetting.getNew(dc, ucs.getUser(), ucs.getClient(), ucs.getName(), ucs.getValue());
			dc.saveItem(duplicate);
			dc.commit();
			dc.deleteItem(duplicate);
			dc.commit();
			throw new BaseException("Expected DatabaseException");
		}
		catch (DatabaseException ex)
		{
			write("--Duplicate user client setting OK");
		}
		catch (Throwable ex)
		{
			write("--Duplicate user client setting FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

}


