/**
	$Id: RootTest.java 4480 2008-09-05 15:16:58Z jari $

	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	BASE is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
	02111-1307, USA. 
*/
package net.sf.basedb.test.performance;

import java.util.List;

import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Experiment;
import net.sf.basedb.core.Formula;
import net.sf.basedb.core.ItemParameterType;
import net.sf.basedb.core.ItemQuery;
import net.sf.basedb.core.Job;
import net.sf.basedb.core.PluginDefinition;
import net.sf.basedb.core.RawBioAssay;
import net.sf.basedb.core.RawDataType;
import net.sf.basedb.core.StringParameterType;
import net.sf.basedb.test.PluginUtil;
import net.sf.basedb.test.TestUtil;
import net.sf.basedb.util.Values;

/**
	Create a root bioassayset.

	@author nicklas
	@version 2.0
	@base.modified $Date: 2008-09-05 17:16:58 +0200 (Fri, 05 Sep 2008) $
*/
public class RootTest
{

	public static boolean ok = true;
		
	/**
		Run the tests in this class.
	*/
	public static boolean run(String user, String password, int numItems)
	{
		DbControl dc = null;
		TestUtil.write("++Entering root bioassayset test\n");
		try
		{
			TestUtil.login(user, password, false);

			// Create raw bioassays
			dc = TestUtil.getDbControl();
			
			// Load prepared items
			Experiment experiment = PrepareTest.getExperiment(dc);
			RawDataType rawDataType = experiment.getRawDataType();
			Formula formula = Util.findFormula(dc, rawDataType, 
				Formula.Type.INTENSITY_EXPRESSION, Config.getIntensityFormula(rawDataType.getId()));
			List<String> expressions = formula.getFormulas();
			long num = experiment.getBioAssaySets().count(dc) + 1;
			dc.close();

			long time = createRootBioAssaySet(experiment, formula, "Root #" + num, numItems) / 1000;
			
			TestUtil.write("---------------------------------------------\n");
			TestUtil.write("formula=" + formula.getName()+"\n");
			TestUtil.write("  ch(1)=" + expressions.get(0) + "\n");
			TestUtil.write("  ch(2)=" + expressions.get(1) + "\n");
			TestUtil.write("Create root bioassayset with " + numItems + " raw data sets: " + 
					Values.formatTime(time) + "\n");
			TestUtil.write("---------------------------------------------\n");
		
			TestUtil.logout();
		}
		catch (Throwable t)
		{
			ok = false;
			t.printStackTrace();
		}
		finally
		{
			if (dc != null) dc.close();
		}
		TestUtil.write("++Leaving root bioassayset test (" + (ok ? "OK" : "FAILED") + ")\n");
		return ok;
	}

	/**
		Create a root bioassay set.
	*/
	public static long createRootBioAssaySet(Experiment experiment, Formula formula, String name, int numRaw)
		throws Exception
	{
		TestUtil.write("--Create root bioassay set: " + name + "\n");
		DbControl dc = TestUtil.getDbControl();
		try
		{
			ItemQuery<RawBioAssay> sources = experiment.getRawBioAssays();
			sources.setMaxResults(numRaw);
			
			PluginDefinition plugin = PluginDefinition.getByClassName(dc, "net.sf.basedb.plugins.IntensityCalculatorPlugin");
			Job job = Job.getNew(dc, plugin, null, experiment);
			job.setName("PerformanceTest: Create root bioassay set: " + name);
			job.setSendMessage(false);
			job.setRemoveJobWhenFinished(true);
			job.setParameterValue("name", 
				new StringParameterType(255, null, true), name);
			job.setParameterValue("experiment", 
				new ItemParameterType<Experiment>(Experiment.class, null), experiment);
			job.setParameterValues("rawBioAssays", 
				new ItemParameterType<RawBioAssay>(RawBioAssay.class, null, true, 0, null), 
				sources.list(dc));
			job.setParameterValue("formula", 
				new ItemParameterType<Formula>(Formula.class, null), formula);
			dc.saveItem(job);
			dc.commit();
			long time = System.currentTimeMillis();
			PluginUtil.executeJob(job);
			time = System.currentTimeMillis() - time;
			return time;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	
}
