/**
	$Id: Run.java 4480 2008-09-05 15:16:58Z jari $

	Copyright (C) 2006, 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	BASE is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
	02111-1307, USA. 
*/
package net.sf.basedb.test.performance;

import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;

import net.sf.basedb.test.CmdLine;
import net.sf.basedb.test.TestUtil;
import net.sf.basedb.util.Values;

/**

	@author nicklas
	@version 2.0
	@base.modified $Date: 2008-09-05 17:16:58 +0200 (Fri, 05 Sep 2008) $
*/
public class Run
{

	public static void main(String[] args)
	{
		
		CmdLine cmdLine = new CmdLine(args);

		Set<String> cmds = new HashSet<String>(Arrays.asList(args));
		boolean all = cmds.contains("all");
		
		String user = cmdLine.getOption("-u", "root");
		String password = cmdLine.getOption("-p", "root");
		int numItems = Values.getInt(cmdLine.getOption("-n"), 4);

		int tests = 0;
		TestUtil.begin(false, true);
		boolean ok = PrepareTest.run(user, password);
		if (ok && (all || cmds.contains("raw")))
		{
			ok = RawDataTest.run(user, password, numItems);
			tests++;
		}
		if (ok && (all || cmds.contains("root")))
		{
			ok = RootTest.run(user, password, numItems);
			tests++;
		}
		if (ok && (all || cmds.contains("filter")))
		{
			ok = FilterTest.run(user, password);
			tests++;
		}
		if (ok && (all || cmds.contains("lowess")))
		{
			ok = LowessTest.run(user, password);
			tests++;
		}
		if (ok && cmds.contains("clean"))
		{
			ok = PrepareTest.cleanUp(user, password);
			tests++;
		}
		TestUtil.stop();
		
		if (tests == 0)
		{
			showUsage();
		}
	}
	
	
	public static void showUsage()
	{
		TestUtil.write(
			"test.sh performance [-n xxx] [-u user] [-p password] <cmd>\n" + 
			"     -n = number of items to import; default = 100\n" +
			"     -u = username; default = root\n" +
			"     -p = password; default = root\n" +
			"     cmd = the tests to run; all, raw\n\n");
		
	}
	
}
