/**
	$Id: Util.java 4480 2008-09-05 15:16:58Z jari $

	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	BASE is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
	02111-1307, USA. 
*/
package net.sf.basedb.test.performance;

import java.util.List;

import net.sf.basedb.core.ArrayDesign;
import net.sf.basedb.core.BioAssaySet;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Experiment;
import net.sf.basedb.core.Formula;
import net.sf.basedb.core.Include;
import net.sf.basedb.core.ItemQuery;
import net.sf.basedb.core.Platform;
import net.sf.basedb.core.RawDataType;
import net.sf.basedb.core.RawDataTypes;
import net.sf.basedb.core.query.Expressions;
import net.sf.basedb.core.query.Hql;
import net.sf.basedb.core.query.Orders;
import net.sf.basedb.core.query.Restrictions;
import net.sf.basedb.test.TestUtil;

/**

	@author nicklas
	@version 2.4
	@base.modified $Date: 2008-09-05 17:16:58 +0200 (Fri, 05 Sep 2008) $
*/
public class Util
{

	/**
		Find the experiment with a matching name. Return null if not found.
	*/
	public static Experiment findExperiment(DbControl dc, String name)
	{
		TestUtil.write("--Loading experiment: " + name + "\n");
		ItemQuery<Experiment> query = Experiment.getQuery();
		query.restrict(Restrictions.eq(Hql.property("name"), Expressions.string(name)));
		List<Experiment> result = query.list(dc);
		return result.isEmpty() ? null : result.get(0);
	}
	
	/**
		Create an experiment.
	*/
	public static Experiment createExperiment(DbControl dc, String name)
	{
		TestUtil.write("--Creating experiment: " + name + "\n");
		Experiment experiment = Experiment.getNew(dc, RawDataTypes.getSafeRawDataType("genepix"));
		experiment.setName(name);
		dc.saveItem(experiment);
		return experiment;
	}
	
	/**
		Find the array design with a matching name. Return null if not found.
	*/
	public static ArrayDesign findArrayDesign(DbControl dc, String name)
	{
		TestUtil.write("--Loading array design: " + name + "\n");
		ItemQuery<ArrayDesign> query = ArrayDesign.getQuery();
		query.restrict(Restrictions.eq(Hql.property("name"), Expressions.string(name)));
		List<ArrayDesign> result = query.list(dc);
		return result.isEmpty() ? null : result.get(0);
	}
	
	/**
		Create an array design.
	*/
	public static ArrayDesign createArrayDesign(DbControl dc, Platform platform, String name)
	{
		TestUtil.write("--Creating array design: " + name + "\n");
		ArrayDesign design = ArrayDesign.getNew(dc, platform);
		design.setName(name);
		dc.saveItem(design);
		return design;
	}

	/**
		Find the formula with a matching name.
	*/
	public static Formula findFormula(DbControl dc, RawDataType rdt, Formula.Type type, String name)
	{
		TestUtil.write("--Loading formula: " + name + "\n");
		ItemQuery<Formula> query = Formula.getQuery(type, rdt);
		query.restrict(Restrictions.eq(Hql.property("name"), Expressions.string(name)));
		query.include(Include.SHARED);
		return query.list(dc).get(0);
	}
	
	/**
		Find a bioassayset with a matching name using LIKE comparison
	*/
	public static BioAssaySet findBioAssaySet(DbControl dc, Experiment experiment, String name)
	{
		TestUtil.write("--Loading bioassayset: " + name);
		ItemQuery<BioAssaySet> query = experiment.getBioAssaySets();
		query.restrict(Restrictions.like(Hql.property("name"), Expressions.string(name)));
		query.order(Orders.desc(Hql.property("id")));
		List<BioAssaySet> result = query.list(dc);
		BioAssaySet bas = result.isEmpty() ? null : result.get(0);
		TestUtil.write(bas == null ? " NOT FOUND\n" : "; found " + bas.getName() + "\n");
		return bas;
	}
	
	
	
}
