/**
	$Id: ProjectService.java 4480 2008-09-05 15:16:58Z jari $

	Copyright (C) 2007 Martin Svensson, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.ws.server;

import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.ItemQuery;
import net.sf.basedb.core.Project;
import net.sf.basedb.core.SessionControl;
import net.sf.basedb.info.ProjectInfo;
import net.sf.basedb.info.QueryOptions;


/**
 	Web service class for Project
 	{@link net.sf.basedb.core.Project}.
  	@author Nicklas, Martin
  	@version 2.5
 */
public class ProjectService
	extends AbstractRPCService
{
	/**
	 	
	 */
	public ProjectService()
	{}
	
	/**
		Get information about a project that you know the ID of.
		@param ID Id to identify the session.
		@param itemId The ID of the project
		@return The project information
	*/
	public ProjectInfo getById(String ID, int itemId)
	{
		SessionControl sc = getSessionControl(ID);
		DbControl dc = sc.newDbControl();	
		try
		{
			Project p = Project.getById(dc, itemId);
			return p.toTransferable(new ProjectInfo());
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	/**
	 	Sets a project active
	  	@param ID Id to identify the active session
	  	@param projectId Id of the project to set as active
	  	@return The Session ID
	 */
	public String setActiveProject(String ID, int projectId)
	{
		SessionControl sc = getSessionControl(ID);
		Project project = null;
		if (projectId != 0)
		{
			DbControl dc = sc.newDbControl();
			try
			{
				project = Project.getById(dc, projectId);
			}
			finally
			{
				if (dc != null) dc.close();
			}
		}
		sc.setActiveProject(project);
		return ID;
	}
	
	/**
	 	Gets information about projects that are available
	 	for an active session.
	  	@param ID Id to identify the session.
	  	@param qOpt QueryOptions to use when getting the projects
	  	@return Information objects in an array for each found project.
	 */
	public ProjectInfo[] getProjects(String ID, QueryOptions qOpt)
	{
		SessionControl sc = getSessionControl(ID);
		DbControl dc = sc.newDbControl();	
		try
		{
			ItemQuery<Project> query = Project.getQuery();
			query = util.getConfiguredItemQuery(query, qOpt);
			return util.listToInfo(query.list(dc), ProjectInfo.class);
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
}
